@@ -71,6 +71,7 @@ class CoalCombustionModel(Variables, Model):
 
         self.coalCombustionModel = ('off', 'homogeneous_fuel', 'homogeneous_fuel_moisture')
 
+        self.y1y2Model = ('user_define', 'automatic_CHONS', 'automatic_formula')
 
     def defaultValues(self):
         """
@@ -133,6 +134,7 @@ class CoalCombustionModel(Variables, Model):
         default['H2O_Kinetics']                          = 'off'
         default['improved_NOx_model']                    = 'off'
         default['reburning']                             = 'unused'
+        default['kinetic_model']                         = 'unused'
 
         return default
 
@@ -241,7 +243,8 @@ class CoalCombustionModel(Variables, Model):
             lst.append("fr_oxyd3")
 
         # ieqco2 fix to true
-        lst.append("x_c_co2")
+        if self.getKineticModel == "co2_ym_transport":
+            lst.append("x_c_co2")
 
         lst.append("f1f2_variance")
 
@@ -1511,7 +1514,7 @@ class CoalCombustionModel(Variables, Model):
         Set Y1Y2 value for a fuel
         """
         self.isInList(str(fuelId), self.getFuelIdList())
-        self.isInList(choice, ('user_define', 'automatic_CHONS', 'automatic_formula'))
+        self.isInList(choice, self.y1y2Model)
         solid_fuel = self.node_fuel.xmlGetNode('solid_fuel', fuel_id = fuelId)
         node = solid_fuel.xmlInitNode('stoichiometric_coefficient')
         node['type'] = choice
@@ -1525,7 +1528,7 @@ class CoalCombustionModel(Variables, Model):
         Return Y1 stoichiometric coefficient for a fuel
         """
         self.isInList(str(fuelId), self.getFuelIdList())
-        self.isInList(self.getY1Y2(fuelId),('user_define', 'automatic_formula'))
+        self.isInList(self.getY1Y2(fuelId), self.y1y2Model)
         solid_fuel = self.node_fuel.xmlGetNode('solid_fuel', fuel_id = fuelId)
         devolatilisation = solid_fuel.xmlInitNode('devolatilisation_parameters')
         node = devolatilisation.xmlInitNode('stoichiometric_coefficient')
@@ -1546,7 +1549,7 @@ class CoalCombustionModel(Variables, Model):
         Set Y1 stoichiometric coefficient for a fuel
         """
         self.isInList(str(fuelId), self.getFuelIdList())
-        self.isInList(self.getY1Y2(fuelId),('user_define', 'automatic_formula'))
+        self.isInList(self.getY1Y2(fuelId), self.y1y2Model)
         self.isPositiveFloat(value)
         solid_fuel = self.node_fuel.xmlGetNode('solid_fuel', fuel_id = fuelId)
         devolatilisation = solid_fuel.xmlInitNode('devolatilisation_parameters')
@@ -1560,7 +1563,7 @@ class CoalCombustionModel(Variables, Model):
         Return Y2 stoichiometric coefficient for a fuel
         """
         self.isInList(str(fuelId), self.getFuelIdList())
-        self.isInList(self.getY1Y2(fuelId),('user_define', 'automatic_formula'))
+        self.isInList(self.getY1Y2(fuelId), self.y1y2Model)
         solid_fuel = self.node_fuel.xmlGetNode('solid_fuel', fuel_id = fuelId)
         devolatilisation = solid_fuel.xmlInitNode('devolatilisation_parameters')
         node = devolatilisation.xmlInitNode('stoichiometric_coefficient')
@@ -1581,7 +1584,7 @@ class CoalCombustionModel(Variables, Model):
         Set Y2 stoichiometric coefficient for a fuel
         """
         self.isInList(str(fuelId), self.getFuelIdList())
-        self.isInList(self.getY1Y2(fuelId),('user_define', 'automatic_formula'))
+        self.isInList(self.getY1Y2(fuelId), self.y1y2Model)
         self.isPositiveFloat(value)
         solid_fuel = self.node_fuel.xmlGetNode('solid_fuel', fuel_id = fuelId)
         devolatilisation = solid_fuel.xmlInitNode('devolatilisation_parameters')
@@ -1962,6 +1965,29 @@ class CoalCombustionModel(Variables, Model):
         return status
 
 
+    @Variables.noUndo
+    def getKineticModel(self):
+        """
+        """
+        value = self.node_fuel.xmlGetString("kinetic_model")
+        if value == '':
+            value = self.defaultValues()['kinetic_model']
+            self.setKineticModel(value)
+        return value
+
+
+    @Variables.undoLocal
+    def setKineticModel(self, choice):
+        """
+        """
+        self.isInList(choice, ('unused',
+                               'co2_ym_transport',
+                               'co_ym_transport'))
+        if choice !=  self.defaultValues()['kinetic_model']:
+            self.node_fuel.xmlSetData('kinetic_model', choice)
+        else:
+            self.node_fuel.xmlRemoveChild('kinetic_model')
+
 #-------------------------------------------------------------------------------
 # SolidFuel combustion test case
 #-------------------------------------------------------------------------------
