@@ -390,20 +390,6 @@ class Package:
 
         os.chdir(os.path.join(build_dir, 'src'))
 
-        # Work around Ubuntu Metis build bug
-        ldflags_add = ''
-        try:
-            p = subprocess.Popen(self.cc + ' -Xlinker --help',
-                                 shell=True,
-                                 universal_newlines=True,
-                                 stdout=subprocess.PIPE,
-                                 stderr=subprocess.PIPE)
-            output = p.communicate()[0]
-            if output.find("--no-as-needed") > -1:
-                ldflags_add = ' -Wl,--no-as-needed\n'
-        except Exception:
-            pass
-
         if self.shared:
             fdr = open('Make.inc/Makefile.inc.x86-64_pc_linux2.shlib')
         else:
@@ -541,7 +527,7 @@ class Setup:
         self.top_srcdir = os.path.abspath(os.path.dirname(sys.argv[0]))
 
         # Optional libraries
-        self.optlibs = ['hdf5', 'cgns', 'med', 'scotch', 'parmetis', 'libxml2']
+        self.optlibs = ['hdf5', 'cgns', 'med', 'scotch', 'parmetis']
 
         # Optional libraries configure could find in salome
         self.salome_optlibs = ['hdf5', 'cgns', 'med']
@@ -598,7 +584,7 @@ class Setup:
                     package="code_saturne",
                     version=None,
                     archive=None,
-                    url="http://code-saturne.org")
+                    url="https://code-saturne.org")
 
         p = self.packages['code_saturne']
 
@@ -613,9 +599,9 @@ class Setup:
             Package(name="HDF5",
                     description="Hierarchical Data Format",
                     package="hdf5",
-                    version="1.8.17",
-                    archive="hdf5-1.8.17.tar.gz",
-                    url="https://support.hdfgroup.org/ftp/HDF5/releases/hdf5-1.8/hdf5-1.8.17/src/%s")
+                    version="1.8.20",
+                    archive="hdf5-1.8.20.tar.gz",
+                    url="https://support.hdfgroup.org/ftp/HDF5/releases/hdf5-1.8/hdf5-1.8.20/src/%s")
 
         p = self.packages['hdf5']
         p.config_opts = "--enable-production"
@@ -639,26 +625,13 @@ class Setup:
             Package(name="MED",
                     description="Model for Exchange of Data",
                     package="med",
-                    version="3.2.1",
-                    archive="med-3.2.1.tar.gz",
+                    version="3.3.1",
+                    archive="med-3.3.1.tar.gz",
                     url="http://files.salome-platform.org/Salome/other/%s")
 
         p = self.packages['med']
         p.config_opts = "--with-med_int=long --disable-fortran --disable-python"
 
-        # Libxml2 library (possible mirror at "ftp://fr.rpmfind.net/pub/libxml/%s")
-
-        self.packages['libxml2'] = \
-            Package(name="libxml2",
-                    description="XML library",
-                    package="libxml2",
-                    version="2.9.2",
-                    archive="libxml2-sources-2.9.2.tar.gz",
-                    url="ftp://xmlsoft.org/libxml2/%s")
-
-        p = self.packages['libxml2']
-        p.config_opts = "--with-ftp=no --with-http=no"
-
         # ParMETIS
 
         self.packages['parmetis'] = \
@@ -675,17 +648,17 @@ class Setup:
             Package(name="scotch",
                     description="PT-Scotch",
                     package="scotch",
-                    version="6.0.4",
-                    archive="scotch_6.0.4.tar.gz",
-                    url="https://gforge.inria.fr/frs/download.php/file/34618/%s")
+                    version="6.0.6",
+                    archive="scotch_6.0.6.tar.gz",
+                    url="https://gforge.inria.fr/frs/download.php/file/37622/%s")
 
     #---------------------------------------------------------------------------
 
     def setup_defaults(self):
 
-        self.cc = find_executable(['cc', 'gcc', 'icc', 'xlc'], 'CC')
+        self.cc = find_executable(['cc', 'gcc', 'icc', 'xlc', 'clang'], 'CC')
         self.fc = find_executable(['f95', 'gfortran', 'ifort'], 'FC')
-        self.cxx = find_executable(['c++', 'g++', 'icpc', 'xlc++'], 'CXX')
+        self.cxx = find_executable(['c++', 'g++', 'icpc', 'xlc++', 'clang++'], 'CXX')
         self.mpicc = find_executable(['mpicc', 'mpicc.openmpi', 'mpicc.mpich'])
         self.mpicxx = find_executable(['mpicxx', 'mpicxx.openmpi', 'mpicxx.mpich'])
         self.python = find_executable(['python'], 'PYTHON')
@@ -720,12 +693,6 @@ class Setup:
                        header='med.h',
                        libname='medC')
 
-        # Libxml2 library (possible mirror at "ftp://fr.rpmfind.net/pub/libxml/%s")
-        p = self.packages['libxml2']
-
-        p.test_library(header='libxml2/libxml/parser.h',
-                       libname='xml2')
-
         # Expand user variables
 
         p = self.packages['code_saturne']
@@ -1055,7 +1022,6 @@ Check the setup file and some utilities presence.
         med= self.packages['med']
         scotch = self.packages['scotch']
         parmetis = self.packages['parmetis']
-        libxml2 = self.packages['libxml2']
 
         # Disable GUI
 
@@ -1077,8 +1043,7 @@ Check the setup file and some utilities presence.
                 config_opts = config_opts + " --with-hdf5=" + hdf5.install_dir
                 med.config_opts += " --with-hdf5=" + hdf5.install_dir
                 cgns.config_opts += "-DCMAKE_PREFIX_PATH=" + hdf5.install_dir \
-                    + " -DHDF5_INCLUDE_PATH=" + hdf5.install_dir + "/include" \
-                    + " -DHDF5_LIBRARY=" + hdf5.install_dir + "/lib/libhdf5.so"
+                    + " -DHDF5_INCLUDE_PATH=" + hdf5.install_dir + "/include"
 
         # CGNS
 
@@ -1110,15 +1075,6 @@ Check the setup file and some utilities presence.
         else:
             config_opts = config_opts + " --with-scotch=" + scotch.install_dir
 
-        # Libxml2
-
-        if libxml2.use == 'no':
-            config_opts = config_opts + " --without-libxml2"
-        else:
-            if libxml2.install_dir:
-                config_opts = config_opts + \
-                    " --with-libxml2=" + libxml2.install_dir
-
         # Python
 
         if self.python:
@@ -1308,11 +1264,7 @@ salome    %(salome)s
 #
 #   To install CGNS or ParMetis, the CMake
 # configuration/installation tool is required
-# (it is available in most Linux distributions)
-#
-#   Libxml2 is needed to read XML files output by the
-# Graphical User Interface. It is generally available
-# through the package manager.
+# (it is available in most Linux distributions).
 #--------------------------------------------------------
 #
 #  Name    Use   Install  Path
