@@ -383,9 +383,10 @@ cs_matrix_wrapper_vector(int                  iconvp,
                          int                  tensorial_diffusion,
                          int                  ndircp,
                          int                  isym,
+                         int                  eb_size[4],
                          double               thetap,
-                         const cs_real_33_t   coefbu[],
-                         const cs_real_33_t   cofbfu[],
+                         const cs_real_33_t   coefbp[],
+                         const cs_real_33_t   cofbfp[],
                          const cs_real_33_t   fimp[],
                          const cs_real_t      i_massflux[],
                          const cs_real_t      b_massflux[],
@@ -410,7 +411,7 @@ cs_matrix_wrapper_vector(int                  iconvp,
       cs_sym_matrix_vector(m,
                            idiffp,
                            thetap,
-                           cofbfu,
+                           cofbfp,
                            fimp,
                            i_visc,
                            b_visc,
@@ -420,11 +421,13 @@ cs_matrix_wrapper_vector(int                  iconvp,
     /* Non-symmetric matrix */
     } else {
       cs_matrix_vector(m,
+                       mq,
                        iconvp,
                        idiffp,
+                       eb_size,
                        thetap,
-                       coefbu,
-                       cofbfu,
+                       coefbp,
+                       cofbfp,
                        fimp,
                        i_massflux,
                        b_massflux,
@@ -442,7 +445,7 @@ cs_matrix_wrapper_vector(int                  iconvp,
       cs_sym_matrix_anisotropic_diffusion(m,
                                           idiffp,
                                           thetap,
-                                          cofbfu,
+                                          cofbfp,
                                           fimp,
                                           (const cs_real_33_t *)i_visc,
                                           b_visc,
@@ -452,11 +455,12 @@ cs_matrix_wrapper_vector(int                  iconvp,
     /* Non-symmetric matrix */
     } else {
       cs_matrix_anisotropic_diffusion(m,
+                                      mq,
                                       iconvp,
                                       idiffp,
                                       thetap,
-                                      coefbu,
-                                      cofbfu,
+                                      coefbp,
+                                      cofbfp,
                                       fimp,
                                       i_massflux,
                                       b_massflux,
@@ -996,7 +1000,7 @@ cs_matrix_scalar(const cs_mesh_t          *m,
  *                               scheme (mix between Crank-Nicolson and
  *                               Adams-Bashforth)
  *                               - thetap = 1: implicit scheme
- * \param[in]     cofbfu        boundary condition array for the variable flux
+ * \param[in]     cofbfp        boundary condition array for the variable flux
  *                               (implicit part - 3x3 tensor array)
  * \param[in]     fimp          \f$ \tens{f_s}^{imp} \f$
  * \param[in]     i_visc        \f$ \mu_\fij \dfrac{S_\fij}{\ipf \jpf} \f$
@@ -1012,7 +1016,7 @@ void
 cs_sym_matrix_vector(const cs_mesh_t          *m,
                      int                       idiffp,
                      double                    thetap,
-                     const cs_real_33_t        cofbfu[],
+                     const cs_real_33_t        cofbfp[],
                      const cs_real_33_t        fimp[],
                      const cs_real_t           i_visc[],
                      const cs_real_t           b_visc[],
@@ -1088,7 +1092,7 @@ cs_sym_matrix_vector(const cs_mesh_t          *m,
     for (int isou = 0; isou < 3; isou++) {
       for (int jsou = 0; jsou < 3; jsou++) {
         da[ii][jsou][isou] += thetap*idiffp*b_visc[face_id]
-                                    *cofbfu[face_id][jsou][isou];
+                                    *cofbfp[face_id][jsou][isou];
       }
     }
 
@@ -1229,6 +1233,7 @@ cs_sym_matrix_tensor(const cs_mesh_t          *m,
  * faces).
  *
  * \param[in]     m             pointer to mesh structure
+ * \param[in]     mq            pointer to mesh quantities structure
  * \param[in]     iconvp        indicator
  *                               - 1 advection
  *                               - 0 otherwise
@@ -1241,9 +1246,9 @@ cs_sym_matrix_tensor(const cs_mesh_t          *m,
  *                               scheme (mix between Crank-Nicolson and
  *                               Adams-Bashforth)
  *                               - thetap = 1: implicit scheme
- * \param[in]     coefbu        boundary condition array for the variable
+ * \param[in]     coefbp        boundary condition array for the variable
  *                               (implicit part - 3x3 tensor array)
- * \param[in]     cofbfu        boundary condition array for the variable flux
+ * \param[in]     cofbfp        boundary condition array for the variable flux
  *                               (implicit part - 3x3 tensor array)
  * \param[in]     fimp          part of the diagonal
  * \param[in]     i_massflux    mass flux at interior faces
@@ -1258,19 +1263,21 @@ cs_sym_matrix_tensor(const cs_mesh_t          *m,
 /*----------------------------------------------------------------------------*/
 
 void
-cs_matrix_vector(const cs_mesh_t          *m,
-                 int                       iconvp,
-                 int                       idiffp,
-                 double                    thetap,
-                 const cs_real_33_t        coefbu[],
-                 const cs_real_33_t        cofbfu[],
-                 const cs_real_33_t        fimp[],
-                 const cs_real_t           i_massflux[],
-                 const cs_real_t           b_massflux[],
-                 const cs_real_t           i_visc[],
-                 const cs_real_t           b_visc[],
-                 cs_real_33_t    *restrict da,
-                 cs_real_2_t     *restrict xa)
+cs_matrix_vector(const cs_mesh_t            *m,
+                 const cs_mesh_quantities_t *mq,
+                 int                         iconvp,
+                 int                         idiffp,
+                 int                         eb_size[4],
+                 double                      thetap,
+                 const cs_real_33_t          coefbp[],
+                 const cs_real_33_t          cofbfp[],
+                 const cs_real_33_t          fimp[],
+                 const cs_real_t             i_massflux[],
+                 const cs_real_t             b_massflux[],
+                 const cs_real_t             i_visc[],
+                 const cs_real_t             b_visc[],
+                 cs_real_33_t      *restrict da,
+                 cs_real_2_t       *restrict xa)
 {
   const cs_lnum_t n_cells = m->n_cells;
   const cs_lnum_t n_cells_ext = m->n_cells_with_ghosts;
@@ -1282,6 +1289,31 @@ cs_matrix_vector(const cs_mesh_t          *m,
   const cs_lnum_t *restrict b_face_cells
     = (const cs_lnum_t *restrict)m->b_face_cells;
 
+  const cs_real_2_t *i_f_face_factor;
+  const cs_real_t *b_f_face_factor;
+
+  /* Discontinuous porous treatment */
+  cs_real_2_t _i_f_face_factor = {1., 1.};
+  cs_real_t _b_f_face_factor = 1.;
+  int is_p = 0;
+
+  if (cs_glob_porous_model == 3) {
+    i_f_face_factor = mq->i_f_face_factor;
+    b_f_face_factor = mq->b_f_face_factor;
+    is_p = 1;
+  }
+  else {
+    i_f_face_factor = _i_f_face_factor;
+    b_f_face_factor = &_b_f_face_factor;
+  }
+
+  const cs_real_3_t *restrict i_face_normal
+    = (const cs_real_3_t *restrict)mq->i_face_normal;
+  const cs_real_3_t *restrict b_face_normal
+    = (const cs_real_3_t *restrict)mq->b_face_normal;
+
+
+  cs_real_332_t *_xa = (cs_real_332_t *) xa;//FIXME why 332 and use 233...
   /* 1. Initialization */
 
   for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
@@ -1306,42 +1338,126 @@ cs_matrix_vector(const cs_mesh_t          *m,
     }
   }
 
-  for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
-    xa[face_id][0] = 0.;
-    xa[face_id][1] = 0.;
+  if (eb_size[0] == 1) {
+    for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
+      xa[face_id][0] = 0.;
+      xa[face_id][1] = 0.;
+    }
+  } else {
+    for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
+      for (cs_lnum_t i = 0; i < eb_size[0]; i++) {
+        for (cs_lnum_t j = 0; j < eb_size[1]; j++) {
+          _xa[face_id][0][i][j] = 0.;
+          _xa[face_id][1][i][j] = 0.;
+        }
+      }
+    }
   }
 
   /* 2. Computation of extradiagonal terms */
 
-  for (cs_lnum_t face_id = 0; face_id <n_i_faces; face_id++) {
+  if (eb_size[0] == 1) {
+    for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
 
-    double flui = 0.5*( i_massflux[face_id] -fabs(i_massflux[face_id]) );
-    double fluj =-0.5*( i_massflux[face_id] +fabs(i_massflux[face_id]) );
+      /*
+       * X_ij = - theta f_j (m_ij)^-
+       * X_ji = - theta f_i (m_ij)^+
+       */
+      cs_real_2_t flu = {
+        0.5 * iconvp * (i_massflux[face_id] - fabs(i_massflux[face_id])),
+        -0.5 * iconvp * (i_massflux[face_id] + fabs(i_massflux[face_id]))
+      };
 
-    xa[face_id][0] = thetap*(iconvp*flui -idiffp*i_visc[face_id]);
-    xa[face_id][1] = thetap*(iconvp*fluj -idiffp*i_visc[face_id]);
+      xa[face_id][0] = thetap*(flu[0] -idiffp*i_visc[face_id])
+        * i_f_face_factor[is_p*face_id][1];//FIXME also diffusion? MF thinks so
+      xa[face_id][1] = thetap*(flu[1] -idiffp*i_visc[face_id])
+        * i_f_face_factor[is_p*face_id][0];
 
+    }
+  }
+  else {
+    for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
+
+      /*
+       * X_ij = - theta f_j (m_ij)^-
+       * X_ji = - theta f_i (m_ij)^+
+       */
+      cs_real_2_t flu = {
+        0.5 * iconvp * (i_massflux[face_id] - fabs(i_massflux[face_id]))
+          - idiffp*i_visc[face_id],
+        -0.5 * iconvp * (i_massflux[face_id] + fabs(i_massflux[face_id]))
+          - idiffp*i_visc[face_id]
+      };
+
+      cs_real_3_t normal;
+      cs_math_3_normalise(i_face_normal[face_id], normal);
+      /* Diagonal part:
+       * the n(x)n term is multiplied by i_f_face_factor and (1 - n(x)n) by 1
+       * XA_ij <= XA_ik n_k n_j (factor - 1) + XA_ij
+       * XA_ij used to be diagonal: XA_ik n_k n_j = XA_ii n_i n_j*/
+      for (cs_lnum_t i = 0; i < eb_size[0]; i++) {
+        _xa[face_id][0][i][i] = flu[0];
+        _xa[face_id][1][i][i] = flu[1];
+        for (cs_lnum_t j = 0; j < eb_size[1]; j++) {
+          _xa[face_id][0][i][j] = thetap*(_xa[face_id][0][i][j]
+              + flu[0]
+              * (i_f_face_factor[is_p*face_id][1] - 1.) * normal[i] * normal[j]);//FIXME also diffusion? MF thinks so
+          _xa[face_id][1][i][j] = thetap*(_xa[face_id][1][i][j]
+              + flu[1]
+              * (i_f_face_factor[is_p*face_id][0] - 1.) * normal[i] * normal[j]);
+        }
+      }
+
+    }
   }
 
-  /* 3. Contribution of the extra-diagonal terms to the diagonal */
+    /* 3. Contribution of the extra-diagonal terms to the diagonal */
 
-  for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
+  if (eb_size[0] == 1) {
+    for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
 
-    cs_lnum_t ii = i_face_cells[face_id][0];
-    cs_lnum_t jj = i_face_cells[face_id][1];
+      cs_lnum_t ii = i_face_cells[face_id][0];
+      cs_lnum_t jj = i_face_cells[face_id][1];
 
-    /* D_ii =  theta (m_ij)^+ - m_ij
-     *      = -X_ij - (1-theta)*m_ij
-     * D_jj = -theta (m_ij)^- + m_ij
-     *      = -X_ji + (1-theta)*m_ij
-     */
-    for (int isou = 0; isou < 3; isou++) {
-      da[ii][isou][isou] -= xa[face_id][0]
-                          + iconvp*(1. - thetap)*i_massflux[face_id];
-      da[jj][isou][isou] -= xa[face_id][1]
-                          - iconvp*(1. - thetap)*i_massflux[face_id];
+      /* D_ii =  theta f_i (m_ij)^+ - m_ij
+       *      = -X_ij - (1-theta)*m_ij
+       *      = -X_ji - m_ij
+       * D_jj = -theta f_j (m_ij)^- + m_ij
+       *      = -X_ji + (1-theta)*m_ij
+       *      = -X_ij + m_ij
+       */
+      for (int i = 0; i < 3; i++) {
+        da[ii][i][i] -= xa[face_id][1]
+                      + iconvp*i_massflux[face_id];
+        da[jj][i][i] -= xa[face_id][0]
+                      - iconvp*i_massflux[face_id];
+      }
     }
+  }
+  else {
+    for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
 
+      cs_lnum_t ii = i_face_cells[face_id][0];
+      cs_lnum_t jj = i_face_cells[face_id][1];
+
+      /* D_ii =  theta (m_ij)^+ - m_ij
+       *      = -X_ij - (1-theta)*m_ij
+       *      = -X_ji - m_ij
+       * D_jj = -theta (m_ij)^- + m_ij
+       *      = -X_ji + (1-theta)*m_ij
+       *      = -X_ij + m_ij
+       */
+      for (cs_lnum_t i = 0; i < eb_size[0]; i++) {
+        da[ii][i][i] -= iconvp * i_massflux[face_id];
+        da[jj][i][i] += iconvp * i_massflux[face_id];
+
+        for (cs_lnum_t j = 0; j < eb_size[1]; j++) {
+          da[ii][i][j] -= _xa[face_id][1][i][j];
+          da[jj][i][j] -= _xa[face_id][0][i][j];
+        }
+      }
+
+    }
   }
 
   /* 4. Contribution of border faces to the diagonal */
@@ -1349,25 +1465,38 @@ cs_matrix_vector(const cs_mesh_t          *m,
   for (cs_lnum_t face_id = 0; face_id < n_b_faces; face_id++) {
 
     cs_lnum_t ii = b_face_cells[face_id];
-    double flui = 0.5*(b_massflux[face_id] -fabs(b_massflux[face_id]));
 
-    for (int isou = 0; isou < 3; isou++) {
-      for (int jsou = 0; jsou < 3; jsou++) {
-        /* D_ii = theta (m_f)^+ + theta B (m_f)^- - m_f
+    cs_real_2_t flu = {
+      /* (m_ij)^+ */
+      iconvp * 0.5 * (b_massflux[face_id] + fabs(b_massflux[face_id])),
+      /* (m_ij)^- */
+      iconvp * 0.5 * (b_massflux[face_id] - fabs(b_massflux[face_id]))
+    };
+
+    cs_real_3_t normal;
+    cs_math_3_normalise(b_face_normal[face_id], normal);
+    cs_real_t n_b_n
+      = cs_math_3_33_3_dot_product(normal, coefbp[face_id], normal);
+    cs_real_t n_bf_n
+      = cs_math_3_33_3_dot_product(normal, cofbfp[face_id], normal);
+
+    for (int i = 0; i < 3; i++) {
+      for (int j = 0; j < 3; j++) {
+        cs_real_t d_ij = ((i == j) ? 1. : 0.);
+        /* D = theta (m_f)^+.1 + theta B (m_f)^- - m_f.1
+         * NB: stop here because the first two terms maybe scaled
          *      = (theta B -1)*(m_f)^- - (1-theta)*(m_f)^+
          *      = theta*(B -1)*(m_f)^- - (1-theta)*m_f
          */
-        if(isou == jsou) {
-          da[ii][jsou][isou] += iconvp*( thetap*flui
-                                        *(coefbu[face_id][jsou][isou]-1.)
-                                       - (1. - thetap)*b_massflux[face_id])
-                              + idiffp*thetap*b_visc[face_id]
-                                      *cofbfu[face_id][jsou][isou];
-        } else {
-          da[ii][jsou][isou] += thetap*( iconvp*flui*coefbu[face_id][jsou][isou]
-                                        +idiffp*b_visc[face_id]
-                                         *cofbfu[face_id][jsou][isou] );
-        }
+          da[ii][i][j] +=
+            thetap * (
+                d_ij * flu[0]
+                + flu[1] * coefbp[face_id][i][j]
+                + idiffp * b_visc[face_id] * cofbfp[face_id][i][j]
+                + (flu[0] + flu[1] * n_b_n + idiffp * b_visc[face_id] * n_bf_n)
+                * (b_f_face_factor[is_p*face_id] - 1.) * normal[i] * normal[j]
+                )
+            - iconvp * d_ij * b_massflux[face_id];
       }
     }
 
@@ -1375,7 +1504,6 @@ cs_matrix_vector(const cs_mesh_t          *m,
 
 }
 
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Build the advection/diffusion matrix for a tensor field
@@ -1694,6 +1822,7 @@ cs_matrix_time_step(const cs_mesh_t          *m,
  * faces).
  *
  * \param[in]     m             pointer to mesh structure
+ * \param[in]     mq            pointer to mesh quantities structure
  * \param[in]     iconvp        indicator
  *                               - 1 advection
  *                               - 0 otherwise
@@ -1723,19 +1852,20 @@ cs_matrix_time_step(const cs_mesh_t          *m,
 /*----------------------------------------------------------------------------*/
 
 void
-cs_matrix_anisotropic_diffusion(const cs_mesh_t          *m,
-                                int                       iconvp,
-                                int                       idiffp,
-                                double                    thetap,
-                                const cs_real_33_t        coefbp[],
-                                const cs_real_33_t        cofbfp[],
-                                const cs_real_33_t        fimp[],
-                                const cs_real_t           i_massflux[],
-                                const cs_real_t           b_massflux[],
-                                const cs_real_33_t        i_visc[],
-                                const cs_real_t           b_visc[],
-                                cs_real_33_t    *restrict da,
-                                cs_real_332_t   *restrict xa)
+cs_matrix_anisotropic_diffusion(const cs_mesh_t            *m,
+                                const cs_mesh_quantities_t *mq,
+                                int                         iconvp,
+                                int                         idiffp,
+                                double                      thetap,
+                                const cs_real_33_t          coefbp[],
+                                const cs_real_33_t          cofbfp[],
+                                const cs_real_33_t          fimp[],
+                                const cs_real_t             i_massflux[],
+                                const cs_real_t             b_massflux[],
+                                const cs_real_33_t          i_visc[],
+                                const cs_real_t             b_visc[],
+                                cs_real_33_t      *restrict da,
+                                cs_real_332_t     *restrict xa)
 {
   const cs_lnum_t n_cells = m->n_cells;
   const cs_lnum_t n_cells_ext = m->n_cells_with_ghosts;
@@ -1747,30 +1877,55 @@ cs_matrix_anisotropic_diffusion(const cs_mesh_t          *m,
   const cs_lnum_t *restrict b_face_cells
     = (const cs_lnum_t *restrict)m->b_face_cells;
 
+  const cs_real_2_t *i_f_face_factor;
+  const cs_real_t *b_f_face_factor;
+
+  const cs_real_3_t *restrict i_face_normal
+    = (const cs_real_3_t *restrict)mq->i_face_normal;
+
+  const cs_real_3_t *restrict b_face_normal
+    = (const cs_real_3_t *restrict)mq->b_face_normal;
+
+  /* Discontinuous porous treatment */
+  cs_real_2_t _i_f_face_factor = {1., 1.};
+  cs_real_t _b_f_face_factor = 1.;
+  int is_p = 0;
+
+  /* Is it porous? */
+  if (cs_glob_porous_model == 3) {
+    i_f_face_factor = mq->i_f_face_factor;
+    b_f_face_factor = mq->b_f_face_factor;
+    is_p = 1;
+  }
+  else {
+    i_f_face_factor = _i_f_face_factor;
+    b_f_face_factor = &_b_f_face_factor;
+  }
+
   /* 1. Initialization */
 
   for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
-    for (int isou = 0; isou < 3; isou++) {
-      for (int jsou = 0; jsou < 3; jsou++) {
-        da[cell_id][jsou][isou] = fimp[cell_id][jsou][isou];
+    for (int i = 0; i < 3; i++) {
+      for (int j = 0; j < 3; j++) {
+        da[cell_id][j][i] = fimp[cell_id][j][i];
       }
     }
   }
   if (n_cells_ext > n_cells) {
     for (cs_lnum_t cell_id = n_cells; cell_id < n_cells_ext; cell_id++) {
-      for (int isou = 0; isou < 3; isou++) {
-        for (int jsou = 0; jsou < 3; jsou++) {
-          da[cell_id][jsou][isou] = 0.;
+      for (int i = 0; i < 3; i++) {
+        for (int j = 0; j < 3; j++) {
+          da[cell_id][j][i] = 0.;
         }
       }
     }
   }
 
   for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
-    for (int isou = 0; isou < 3; isou++) {
-      for (int jsou = 0; jsou < 3; jsou++) {
-        xa[face_id][0][jsou][isou] = 0.;
-        xa[face_id][1][jsou][isou] = 0.;
+    for (int i = 0; i < 3; i++) {
+      for (int j = 0; j < 3; j++) {
+        xa[face_id][0][j][i] = 0.;
+        xa[face_id][1][j][i] = 0.;
       }
     }
   }
@@ -1779,16 +1934,25 @@ cs_matrix_anisotropic_diffusion(const cs_mesh_t          *m,
 
   for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
 
-    double flui = 0.5*(i_massflux[face_id] -fabs(i_massflux[face_id]));
-    double fluj =-0.5*(i_massflux[face_id] +fabs(i_massflux[face_id]));
+    cs_real_2_t flu = {
+      iconvp * 0.5*(i_massflux[face_id] -fabs(i_massflux[face_id])),
+      -iconvp * 0.5*(i_massflux[face_id] +fabs(i_massflux[face_id]))
+    };
+
+    cs_real_3_t normal;
+    cs_math_3_normalise(i_face_normal[face_id], normal);
 
     for (int i = 0; i < 3; i++) {
-      xa[face_id][0][i][i] = iconvp*flui;
-      xa[face_id][1][i][i] = iconvp*fluj;
+      xa[face_id][0][i][i] = flu[0];
+      xa[face_id][1][i][i] = flu[1];
       for (int j = 0; j < 3; j++) {
         xa[face_id][0][i][j] = thetap*( xa[face_id][0][i][j]
-                                      - idiffp*i_visc[face_id][i][j]);
+                                      + (i_f_face_factor[is_p*face_id][0] - 1.)
+                                        * normal[i] * normal[j] * flu[0]
+                                      - idiffp*i_visc[face_id][i][j]);//FIXME also diffusion? MF thinks so
         xa[face_id][1][i][j] = thetap*( xa[face_id][1][i][j]
+                                      + (i_f_face_factor[is_p*face_id][1] - 1.)
+                                        * normal[i] * normal[j] * flu[1]
                                       - idiffp*i_visc[face_id][i][j]);
       }
     }
@@ -1822,25 +1986,38 @@ cs_matrix_anisotropic_diffusion(const cs_mesh_t          *m,
   for (cs_lnum_t face_id = 0; face_id < n_b_faces; face_id++) {
 
     cs_lnum_t ii = b_face_cells[face_id];
-    double flui = 0.5*( b_massflux[face_id] -fabs(b_massflux[face_id]) );
+
+    cs_real_2_t flu = {
+      /* (m_ij)^+ */
+      iconvp * 0.5 * (b_massflux[face_id] + fabs(b_massflux[face_id])),
+      /* (m_ij)^- */
+      iconvp * 0.5 * (b_massflux[face_id] - fabs(b_massflux[face_id]))
+    };
+
+    cs_real_3_t normal;
+    cs_math_3_normalise(b_face_normal[face_id], normal);
+    cs_real_t n_b_n
+      = cs_math_3_33_3_dot_product(normal, coefbp[face_id], normal);
+    cs_real_t n_bf_n
+      = cs_math_3_33_3_dot_product(normal, cofbfp[face_id], normal);
 
     for (int i = 0; i < 3; i++) {
       for (int j = 0; j < 3; j++) {
-        /* D_ii = theta (m_f)^+ + theta B (m_f)^- - m_f
+        cs_real_t d_ij = ((i == j) ? 1. : 0.);
+        /* D = theta (m_f)^+.1 + theta B (m_f)^- - m_f.1
+         * NB: stop here because the first two terms maybe scaled
          *      = (theta B -1)*(m_f)^- - (1-theta)*(m_f)^+
          *      = theta*(B -1)*(m_f)^- - (1-theta)*m_f
          */
-        if(j == i) {
-          da[ii][i][j] += iconvp*( thetap*flui
-                                        *(coefbp[face_id][i][j]-1.)
-                                       - (1. - thetap)*b_massflux[face_id])
-                              + idiffp*thetap*b_visc[face_id]
-                                      *cofbfp[face_id][i][j];
-        } else {
-          da[ii][i][j] += thetap*( iconvp*flui*coefbp[face_id][i][j]
-                                        +idiffp*b_visc[face_id]
-                                         *cofbfp[face_id][i][j] );
-        }
+          da[ii][i][j] +=
+            thetap * (
+                d_ij * flu[0]
+                + flu[1] * coefbp[face_id][i][j]
+                + idiffp * b_visc[face_id] * cofbfp[face_id][i][j]
+                + (flu[0] + flu[1] * n_b_n + idiffp * b_visc[face_id] * n_bf_n)
+                * (b_f_face_factor[is_p*face_id] - 1.) * normal[i] * normal[j]
+                )
+            - iconvp * d_ij * b_massflux[face_id];
       }
     }
 
@@ -1871,9 +2048,9 @@ cs_matrix_anisotropic_diffusion(const cs_mesh_t          *m,
  *                               scheme (mix between Crank-Nicolson and
  *                               Adams-Bashforth)
  *                               - thetap = 1: implicit scheme
- * \param[in]     coefbu        boundary condition array for the variable
+ * \param[in]     coefbp        boundary condition array for the variable
  *                               (implicit part - 3x3 tensor array)
- * \param[in]     cofbfu        boundary condition array for the variable flux
+ * \param[in]     cofbfp        boundary condition array for the variable flux
  *                               (implicit part - 3x3 tensor array)
  * \param[in]     fimp          part of the diagonal
  * \param[in]     i_massflux    mass flux at interior faces
@@ -1892,8 +2069,8 @@ cs_matrix_anisotropic_diffusion_tensor(const cs_mesh_t          *m,
                                        int                       iconvp,
                                        int                       idiffp,
                                        double                    thetap,
-                                       const cs_real_66_t        coefbu[],
-                                       const cs_real_66_t        cofbfu[],
+                                       const cs_real_66_t        coefbp[],
+                                       const cs_real_66_t        cofbfp[],
                                        const cs_real_66_t        fimp[],
                                        const cs_real_t           i_massflux[],
                                        const cs_real_t           b_massflux[],
@@ -1998,14 +2175,14 @@ cs_matrix_anisotropic_diffusion_tensor(const cs_mesh_t          *m,
          */
         if(j == i) {
           da[ii][i][j] += iconvp*( thetap*flui
-                                        *(coefbu[face_id][i][j]-1.)
+                                        *(coefbp[face_id][i][j]-1.)
                                        - (1. - thetap)*b_massflux[face_id])
                               + idiffp*thetap*b_visc[face_id]
-                                      *cofbfu[face_id][i][j];
+                                      *cofbfp[face_id][i][j];
         } else {
-          da[ii][i][j] += thetap*( iconvp*flui*coefbu[face_id][i][j]
+          da[ii][i][j] += thetap*( iconvp*flui*coefbp[face_id][i][j]
                                         +idiffp*b_visc[face_id]
-                                         *cofbfu[face_id][i][j] );
+                                         *cofbfp[face_id][i][j] );
         }
       }
     }
@@ -2034,7 +2211,7 @@ cs_matrix_anisotropic_diffusion_tensor(const cs_mesh_t          *m,
  *                               scheme (mix between Crank-Nicolson and
  *                               Adams-Bashforth)
  *                               - thetap = 1: implicit scheme
- * \param[in]     cofbfu        boundary condition array for the variable flux
+ * \param[in]     cofbfp        boundary condition array for the variable flux
  *                               (implicit part - 3x3 tensor array)
  * \param[in]     fimp          \f$ \tens{f_s}^{imp} \f$
  * \param[in]     i_visc        \f$ \mu_\fij \dfrac{S_\fij}{\ipf \jpf} \f$
@@ -2050,7 +2227,7 @@ void
 cs_sym_matrix_anisotropic_diffusion(const cs_mesh_t           *m,
                                     int                       idiffp,
                                     double                    thetap,
-                                    const cs_real_33_t        cofbfu[],
+                                    const cs_real_33_t        cofbfp[],
                                     const cs_real_33_t        fimp[],
                                     const cs_real_33_t        i_visc[],
                                     const cs_real_t           b_visc[],
@@ -2131,10 +2308,10 @@ cs_sym_matrix_anisotropic_diffusion(const cs_mesh_t           *m,
       for (int jsou = 0; jsou < 3; jsou++) {
         if(isou == jsou) {
           da[ii][jsou][isou] += thetap*idiffp*b_visc[ifac]
-                                      *cofbfu[ifac][jsou][isou];
+                                      *cofbfp[ifac][jsou][isou];
         } else {
           da[ii][jsou][isou] += thetap*idiffp*b_visc[ifac]
-                                      *cofbfu[ifac][jsou][isou];
+                                      *cofbfp[ifac][jsou][isou];
         }
       }
     }
@@ -2163,7 +2340,7 @@ cs_sym_matrix_anisotropic_diffusion(const cs_mesh_t           *m,
  *                               scheme (mix between Crank-Nicolson and
  *                               Adams-Bashforth)
  *                               - thetap = 1: implicit scheme
- * \param[in]     cofbfu        boundary condition array for the variable flux
+ * \param[in]     cofbfp        boundary condition array for the variable flux
  *                               (implicit part - 3x3 tensor array)
  * \param[in]     fimp          \f$ \tens{f_s}^{imp} \f$
  * \param[in]     i_visc        \f$ \mu_\fij \dfrac{S_\fij}{\ipf \jpf} \f$
@@ -2179,7 +2356,7 @@ void
 cs_sym_matrix_anisotropic_diffusion_tensor(const cs_mesh_t           *m,
                                            int                       idiffp,
                                            double                    thetap,
-                                           const cs_real_66_t        cofbfu[],
+                                           const cs_real_66_t        cofbfp[],
                                            const cs_real_66_t        fimp[],
                                            const cs_real_66_t        i_visc[],
                                            const cs_real_t           b_visc[],
@@ -2259,7 +2436,7 @@ cs_sym_matrix_anisotropic_diffusion_tensor(const cs_mesh_t           *m,
     for (int isou = 0; isou < 6; isou++) {
       for (int jsou = 0; jsou < 6; jsou++) {
         da[ii][jsou][isou] += thetap*idiffp*b_visc[ifac]
-                                    *cofbfu[ifac][jsou][isou];
+                                    *cofbfp[ifac][jsou][isou];
       }
     }
 
