@@ -1102,8 +1102,8 @@ cs_halo_sync_untyped(const cs_halo_t  *halo,
 #if defined(HAVE_MPI)
 
   if (cs_glob_n_ranks > 1) {
-    const size_t send_buffer_size =   CS_MAX(halo->n_send_elts[CS_HALO_EXTENDED],
-                                             halo->n_elts[CS_HALO_EXTENDED])
+    const size_t send_buffer_size = CS_MAX(halo->n_send_elts[CS_HALO_EXTENDED],
+                                           halo->n_elts[CS_HALO_EXTENDED])
                                     * size;
 
     if (send_buffer_size > _cs_glob_halo_send_buffer_size) {
@@ -1436,9 +1436,9 @@ cs_halo_sync_var(const cs_halo_t  *halo,
 
     }
 
-    /* Assemble buffers for halo exchange;
-       avoid threading for now, as dynamic scheduling led to slightly higher cost here,
-       and even static scheduling might lead to false sharing for small halos. */
+    /* Assemble buffers for halo exchange; avoid threading for now, as dynamic
+       scheduling led to slightly higher cost here, and even static scheduling
+       might lead to false sharing for small halos. */
 
     for (rank_id = 0; rank_id < halo->n_c_domains; rank_id++) {
 
@@ -1585,9 +1585,9 @@ cs_halo_sync_var_strided(const cs_halo_t  *halo,
 
     }
 
-    /* Assemble buffers for halo exchange;
-       avoid threading for now, as dynamic scheduling led to slightly higher cost here,
-       and even static scheduling might lead to false sharing for small halos. */
+    /* Assemble buffers for halo exchange; avoid threading for now, as dynamic
+       scheduling led to slightly higher cost here, and even static scheduling
+       might lead to false sharing for small halos. */
 
     for (rank_id = 0; rank_id < halo->n_c_domains; rank_id++) {
 
@@ -1914,8 +1914,8 @@ cs_halo_dump(const cs_halo_t  *halo,
         bft_printf("  idx start %d:          idx end   %d:\n",
                    index[2*i], index[2*i+1]);
 
-        if (print_level == 1 && list != NULL) {
-          bft_printf("\n            id      cell id\n");
+        if (print_level > 0 && list != NULL) {
+          bft_printf("\n            idx     elt id\n");
           for (j = index[2*i]; j < index[2*i+1]; j++)
             bft_printf("    %10d %10d\n", j, list[j]);
         }
@@ -1928,8 +1928,8 @@ cs_halo_dump(const cs_halo_t  *halo,
         bft_printf("  idx start %d:          idx end   %d:\n",
                    index[2*i+1], index[2*i+2]);
 
-        if (print_level == 1 && list != NULL) {
-          bft_printf("\n            id      cell id\n");
+        if (print_level > 0 && list != NULL) {
+          bft_printf("\n            idx     elt id\n");
           for (j = index[2*i+1]; j < index[2*i+2]; j++)
             bft_printf("    %10d %10d %10d\n",
                        j, list[j], halo->n_local_elts+j);
