@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2018 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -46,6 +46,7 @@
 
 #include "cs_array_reduce.h"
 #include "cs_base.h"
+#include "cs_ctwr.h"
 #include "cs_fan.h"
 #include "cs_field.h"
 #include "cs_log.h"
@@ -54,11 +55,14 @@
 #include "cs_mesh_location.h"
 #include "cs_parall.h"
 #include "cs_prototypes.h"
+#include "cs_range_set.h"
 #include "cs_time_moment.h"
+#include "cs_time_plot.h"
 #include "cs_time_step.h"
 #include "cs_lagr_stat.h"
 #include "cs_lagr_log.h"
 
+
 /*----------------------------------------------------------------------------
  * Header for the current file
  *----------------------------------------------------------------------------*/
@@ -138,14 +142,13 @@ static double *_clips_vmin = NULL;
 static double *_clips_vmax = NULL;
 static cs_log_clip_t  *_clips = NULL;
 
+static cs_time_plot_t  *_l2_residual_plot = NULL;
+
 /*============================================================================
  * Prototypes for functions intended for use only by Fortran wrappers.
  * (descriptions follow, with function bodies).
  *============================================================================*/
 
-void
-cs_f_log_iteration(void);
-
 /*============================================================================
  * Fortran function prototypes for subroutines from field.f90.
  *============================================================================*/
@@ -336,6 +339,7 @@ _find_clip(int  f_id,
  *   vmax         <-- maximum values of each component or norm
  *   vsum         <-- sum of each component or norm
  *   wsum         <-- weighted sum of each component or norm, or NULL
+ *   fpe_flag     <-- was a "not a number" or floating-point error detected ?
  *----------------------------------------------------------------------------*/
 
 static void
@@ -348,14 +352,14 @@ _log_array_info(const char        *prefix,
                 double             vmin[],
                 const double       vmax[],
                 const double       vsum[],
-                const double      *wsum)
+                const double      *wsum,
+                int               *fpe_flag)
 {
-  int c_id;
   const int _dim = (dim == 3) ? 4 : dim;
 
   char tmp_s[2][64] =  {"", ""};
 
-  for (c_id = 0; c_id < _dim; c_id++) {
+  for (int c_id = 0; c_id < _dim; c_id++) {
 
     if (dim == 3) {
       if (c_id < 3)
@@ -396,7 +400,11 @@ _log_array_info(const char        *prefix,
                     vmax[c_id],
                     vsum[c_id] / n_g_elts);
 
+    /* Check NAN  and exit */
+    if (isnan(vsum[c_id]))
+      *fpe_flag = 1;
   }
+
 }
 
 /*----------------------------------------------------------------------------
@@ -508,6 +516,7 @@ _log_fields(void)
   int f_id, li, log_count;
 
   int log_count_max = 0;
+  int fpe_flag = 0;
   int     *log_id = NULL, *moment_id = NULL;
   double  *vmin = NULL, *vmax = NULL, *vsum = NULL, *wsum = NULL;
 
@@ -525,7 +534,7 @@ _log_fields(void)
                                          CS_MESH_LOCATION_VERTICES
                                          };
 
-  const cs_mesh_t *m = cs_glob_mesh;
+  cs_mesh_t *m = cs_glob_mesh;
   const cs_mesh_quantities_t *mq = cs_glob_mesh_quantities;
 
   /* Allocate working arrays */
@@ -558,6 +567,7 @@ _log_fields(void)
     int have_weight = 0;
     double total_weight = -1;
     cs_gnum_t n_g_elts = 0;
+    cs_real_t *gather_array = NULL; /* only if CS_MESH_LOCATION_VERTICES */
     const cs_lnum_t *n_elts = cs_mesh_location_get_n_elts(loc_id);
     const cs_lnum_t _n_elts = n_elts[0];
     const cs_real_t *weight = NULL;
@@ -587,6 +597,7 @@ _log_fields(void)
       case CS_MESH_LOCATION_VERTICES:
         n_g_elts = m->n_g_vertices;
         have_weight = 0;
+        BFT_MALLOC(gather_array, m->n_vertices, cs_real_t);
         break;
       default:
         n_g_elts = _n_elts;
@@ -595,6 +606,9 @@ _log_fields(void)
       }
     }
 
+    if (n_g_elts == 0)
+      continue;
+
     /* First loop on fields */
 
     log_count = 0;
@@ -649,10 +663,34 @@ _log_fields(void)
 
       }
       else {
-        cs_array_reduce_simple_stats_l(_n_elts,
+
+        cs_real_t  *field_val = f->val;
+        cs_lnum_t  _n_cur_elts = _n_elts;
+        if (gather_array != NULL) { /* Eliminate shared values whose local
+                                       rank is not owner and compact */
+
+          if (m->vtx_range_set == NULL)
+            m->vtx_range_set = cs_range_set_create(m->vtx_interfaces,
+                                                   NULL,
+                                                   m->n_vertices,
+                                                   false, /* balance */
+                                                   0); /* g_id_base */
+
+          if (f->dim > 1)
+            BFT_REALLOC(gather_array, (f->dim * m->n_vertices), cs_real_t);
+
+          cs_range_set_gather(m->vtx_range_set,
+                              CS_REAL_TYPE,
+                              f->dim,
+                              f->val,
+                              gather_array);
+          field_val = gather_array;
+          _n_cur_elts = m->vtx_range_set->n_elts[0];
+        }
+        cs_array_reduce_simple_stats_l(_n_cur_elts,
                                        f->dim,
                                        NULL,
-                                       f->val,
+                                       field_val,
                                        vmin + log_count,
                                        vmax + log_count,
                                        vsum + log_count);
@@ -679,6 +717,9 @@ _log_fields(void)
 
     } /* End of first loop on fields */
 
+    if (gather_array != NULL)
+      BFT_FREE(gather_array);
+
     if (log_count < 1)
       continue;
 
@@ -784,7 +825,8 @@ _log_fields(void)
                       vmin + log_count,
                       vmax + log_count,
                       vsum + log_count,
-                      wsum + log_count);
+                      wsum + log_count,
+                      &fpe_flag);
 
       log_count += _dim;
 
@@ -792,6 +834,11 @@ _log_fields(void)
 
   } /* End of loop on mesh locations */
 
+  /* Check NaN and exit */
+  if (fpe_flag == 1)
+    bft_error(__FILE__, __LINE__, 0,
+                _("Invalid (not-a-number) values detected for a field."));
+
   BFT_FREE(moment_id);
   BFT_FREE(wsum);
   BFT_FREE(vsum);
@@ -810,6 +857,7 @@ static void
 _log_sstats(void)
 {
   int     stat_id;
+  int     fpe_flag = 0;
   double _boundary_surf = -1;
   double _interior_surf = -1;
   double  *vmin = NULL, *vmax = NULL, *vsum = NULL, *wsum = NULL;
@@ -1008,7 +1056,8 @@ _log_sstats(void)
                         vmin + stat_id,
                         vmax + stat_id,
                         vsum + stat_id,
-                        wsum + stat_id);
+                        wsum + stat_id,
+                        &fpe_flag);
 
       } /* End of loop on stats */
 
@@ -1018,6 +1067,11 @@ _log_sstats(void)
 
   } /* End of loop on mesh categories */
 
+  /* Check NaN and exit */
+  if (fpe_flag == 1)
+    bft_error(__FILE__, __LINE__, 0,
+                _("Invalid (not-a-number) values detected for a statistic."));
+
   BFT_FREE(wsum);
   BFT_FREE(vsum);
   BFT_FREE(vmax);
@@ -1346,6 +1400,9 @@ cs_log_iteration_destroy_all(void)
 
   if (_name_map != NULL)
     cs_map_name_to_id_destroy(&_name_map);
+
+  if (_l2_residual_plot != NULL)
+    cs_time_plot_finalize(&_l2_residual_plot);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1370,6 +1427,7 @@ cs_log_iteration(void)
   cs_lagr_log_iteration();
 
   cs_fan_log_iteration();
+  cs_ctwr_log_balance();
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1590,6 +1648,80 @@ cs_log_iteration_clipping_field(int               f_id,
                 min_pre_clip, max_pre_clip,n_clip_min_comp,n_clip_max_comp);
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Log L2 time residual for every variable fields.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_log_l2residual(void)
+{
+  const cs_time_step_t *ts = cs_glob_time_step;
+  const int n_fields = cs_field_n_fields();
+
+  /* write header */
+
+  if (_l2_residual_plot == NULL) {
+
+    int                    _plot_buffer_steps = -1;
+    double                 _plot_flush_wtime = 3600;
+    cs_time_plot_format_t  _plot_format = CS_TIME_PLOT_CSV;
+    bool                   use_iteration = (ts->is_local) ? true : false;
+
+    const char **labels;
+    BFT_MALLOC(labels, n_fields + 1, const char *);
+
+    int n_variables = 0;
+    for (int f_id = 0 ; f_id < n_fields ; f_id++) {
+      const cs_field_t *f = cs_field_by_id(f_id);
+      if (f->type & CS_FIELD_VARIABLE) {
+        labels[n_variables] = f->name;
+        n_variables++;
+      }
+    }
+
+    _l2_residual_plot = cs_time_plot_init_probe("residuals",
+                                                "",
+                                                _plot_format,
+                                                use_iteration,
+                                                _plot_flush_wtime,
+                                                _plot_buffer_steps,
+                                                n_variables,
+                                                NULL,
+                                                NULL,
+                                                labels);
+
+    BFT_FREE(labels);
+  }
+
+  {
+    cs_real_t *vals;
+    BFT_MALLOC(vals, n_fields, cs_real_t);
+
+    int si_k_id = cs_field_key_id("solving_info");
+
+    int n_variables = 0;
+    for (int f_id = 0 ; f_id < n_fields ; f_id++) {
+      const cs_field_t *f = cs_field_by_id(f_id);
+      if (f->type & CS_FIELD_VARIABLE) {
+        const cs_solving_info_t *sinfo
+          = cs_field_get_key_struct_const_ptr(f, si_k_id);
+        vals[n_variables] = sinfo->l2residual;
+        n_variables += 1;
+      }
+    }
+
+    cs_time_plot_vals_write(_l2_residual_plot,
+                            ts->nt_cur,
+                            ts->t_cur,
+                            n_variables,
+                            vals);
+
+    BFT_FREE(vals);
+  }
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
