@@ -1372,7 +1372,7 @@ cs_parameters_check(void)
                                 _("while reading input data"),
                                 "cs_glob_wall_functions->iwallf",
                                 cs_glob_wall_functions->iwallf,
-                                0, 7);
+                                0, 8);
 
   if (cs_glob_wall_functions->iwallf > 2) {
     const int itytur_vals[5] = {2, 3, 5, 6, 7};
@@ -1773,15 +1773,6 @@ cs_parameters_check(void)
                                      "number of coupled scalar",
                                      cs_field_get_key_int(f, keysca),
                                      cs_glob_thermal_model->iscalt);
-
-          cs_parameters_is_equal_int(CS_ABORT_DELAYED,
-                                     _("Inconsistency in SYRTHES coupling "
-                                       "settings,\n"
-                                       "the thermal scalar must be "
-                                       "the temperature"),
-                                     "cs_glob_thermal_model->itherm",
-                                     cs_glob_thermal_model->itherm,
-                                     CS_THERMAL_MODEL_TEMPERATURE);
         }
       }
     }
@@ -2038,16 +2029,6 @@ cs_parameters_check(void)
      module */
   int model = cs_turbomachinery_get_model();
 
-  if (model == CS_TURBOMACHINERY_TRANSIENT) {
-    cs_parameters_is_equal_int(CS_ABORT_DELAYED,
-                               _("transient turbomachinery module not "
-                                 "compatible "
-                                 "with the lagrangian module"),
-                               "cs_glob_lagr_time_scheme->iilagr",
-                               cs_glob_lagr_time_scheme->iilagr,
-                               0);
-  }
-
   if (model > CS_TURBOMACHINERY_NONE) {
     cs_parameters_is_equal_int(CS_ABORT_DELAYED,
                                _("turbomachinery module not compatible "
@@ -2060,6 +2041,23 @@ cs_parameters_check(void)
                                0);
   }
 
+  /* Check the consistency of time extrapolation key */
+  for (int f_id = 0 ; f_id < cs_field_n_fields() ; f_id++) {
+    cs_field_t  *f = cs_field_by_id(f_id);
+
+    int t_ext = cs_field_get_key_int(f, cs_field_key_id("time_extrapolated"));
+
+    f_desc = _field_section_desc(f, "while reading parameters for "
+                                    "field ");
+    cs_parameters_is_in_range_int(CS_ABORT_DELAYED,
+                                  _(f_desc),
+                                  "key 'time_extrapolated'",
+                                  t_ext,
+                                  -1,
+                                  3);
+    }
+
+
   /* Check the consistency of the restart_file key */
   for (int f_id = 0 ; f_id < cs_field_n_fields() ; f_id++) {
     cs_field_t  *f = cs_field_by_id(f_id);
