@@ -62,8 +62,7 @@
 !> \param[in]     dt            time step (per cell)
 !> \param[in]     vel           velocity
 !> \param[in]     vela          velocity at the previous time step
-!> \param[in]     flumas        internal mass flux (depending on iappel)
-!> \param[in]     flumab        boundary mass flux (depending on iappel)
+!> \param[in]     velk          velocity at the previous sub iteration (or vela)
 !> \param[in]     tslagr        coupling term for the Lagrangian module
 !> \param[in]     coefav        boundary condition array for the variable
 !>                               (explicit part)
@@ -80,13 +79,8 @@
 !> \param[in]     spcond        variable value associated to the condensation
 !>                              source term (for ivar=ipr, spcond is the flow rate
 !>                              \f$ \Gamma_{s, cond}^n \f$)
-!> \param[in]     svcond        variable value associated to the condensation
-!>                              source term (for ivar=ipr, svcond is the flow rate
-!>                              \f$ \Gamma_{v, cond}^n \f$)
 !> \param[in]     frcxt         external forces making hydrostatic pressure
 !> \param[in]     trava         working array for the velocity-pressure coupling
-!> \param[in]     ximpa         same
-!> \param[in]     uvwk          same (stores the velocity at the previous iteration)
 !> \param[in]     dfrcxt        variation of the external forces
 !                               making the hydrostatic pressure
 !> \param[in]     grdphd        hydrostatic pressure gradient to handle the
@@ -103,9 +97,6 @@
 !> \param[in]     secvif        secondary viscosity at interior faces
 !> \param[in]     secvib        secondary viscosity at boundary faces
 !> \param[in]     w1            working array
-!> \param[in]     w7            working array
-!> \param[in]     w8            working array
-!> \param[in]     w9            working array
 !_______________________________________________________________________________
 
 subroutine predvv &
@@ -114,13 +105,12 @@ subroutine predvv &
    ncepdp , ncesmp , nfbpcd , ncmast ,                            &
    icepdc , icetsm , ifbpcd , ltmast ,                            &
    itypsm ,                                                       &
-   dt     , vel    , vela   ,                                     &
-   flumas , flumab ,                                              &
+   dt     , vel    , vela   , velk   ,                            &
    tslagr , coefav , coefbv , cofafv , cofbfv ,                   &
-   ckupdc , smacel , spcond , svcond , frcxt  , grdphd ,          &
-   trava  , ximpa  , uvwk   , dfrcxt , tpucou , trav   ,          &
+   ckupdc , smacel , spcond , frcxt  , grdphd ,                   &
+   trava  ,                   dfrcxt , tpucou , trav   ,          &
    viscf  , viscb  , viscfi , viscbi , secvif , secvib ,          &
-   w1     , w7     , w8     , w9     )
+   w1     )
 
 !===============================================================================
 
@@ -150,10 +140,8 @@ use cs_c_bindings
 use cfpoin
 use field
 use field_operator
-use pointe, only: gamcav
 use cavitation
 use vof
-use cs_tagms, only:s_metal
 use atincl, only: kopint, iatmst
 
 !===============================================================================
@@ -172,27 +160,25 @@ integer          ifbpcd(nfbpcd)
 integer          ltmast(ncelet)
 
 double precision dt(ncelet)
-double precision flumas(nfac), flumab(nfabor)
 double precision tslagr(ncelet,*)
-double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
-double precision spcond(nfbpcd,nvar), svcond(ncelet,nvar)
+double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
+double precision spcond(nfbpcd,nvar)
 double precision frcxt(3,ncelet), dfrcxt(3,ncelet)
 double precision grdphd(3, ncelet)
 double precision trava(ndim,ncelet)
-double precision ximpa(ndim,ndim,ncelet),uvwk(ndim,ncelet)
 double precision tpucou(6, ncelet)
 double precision trav(3,ncelet)
 double precision viscf(*), viscb(nfabor)
 double precision viscfi(*), viscbi(nfabor)
 double precision secvif(nfac), secvib(nfabor)
 double precision w1(ncelet)
-double precision w7(ncelet), w8(ncelet), w9(ncelet)
 double precision coefav(3  ,nfabor)
 double precision cofafv(3  ,nfabor)
 double precision coefbv(3,3,nfabor)
 double precision cofbfv(3,3,nfabor)
 
 double precision vel   (3, ncelet)
+double precision velk  (3, ncelet)
 double precision vela  (3, ncelet)
 
 ! Local variables
@@ -203,10 +189,11 @@ integer          nswrgp, imligp, iwarnp
 integer          iswdyp, idftnp
 integer          iconvp, idiffp, ndircp, nswrsp
 integer          ircflp, ischcp, isstpp, iescap
-integer          iflmb0, nswrp
+integer          iflmb0
 integer          idtva0, icvflb, f_oi_id
 integer          jsou  , ivisep, imasac
 integer          f_dim , iflwgr
+integer          iflmas, iflmab
 integer          ivoid(1)
 
 double precision rnorm , vitnor
@@ -215,9 +202,8 @@ double precision epsrgp, climgp, extrap, relaxp, blencp, epsilp
 double precision epsrsp
 double precision vit1  , vit2  , vit3, xkb, pip, pfac, pfac1
 double precision cpdc11, cpdc22, cpdc33, cpdc12, cpdc13, cpdc23
-double precision d2s3  , thetap, thetp1, thets , dtsrom
+double precision d2s3  , thetap, thetp1, thets
 double precision diipbx, diipby, diipbz
-double precision ccorio
 double precision dvol
 
 double precision rvoid(1)
@@ -225,6 +211,7 @@ double precision rvoid(1)
 ! Working arrays
 double precision, allocatable, dimension(:,:) :: eswork
 double precision, allocatable, dimension(:,:), target :: grad
+double precision, allocatable, dimension(:,:), target :: hl_exp
 double precision, dimension(:,:), allocatable :: smbr
 double precision, dimension(:,:,:), allocatable :: fimp
 double precision, dimension(:,:), allocatable :: gavinj
@@ -232,8 +219,9 @@ double precision, dimension(:,:), allocatable :: tsexp
 double precision, dimension(:,:,:), allocatable :: tsimp
 double precision, allocatable, dimension(:,:) :: viscce
 double precision, dimension(:,:), allocatable :: vect
-double precision, dimension(:), allocatable :: xinvro
-double precision, dimension(:), pointer :: brom, crom, croma, pcrom
+double precision, dimension(:), pointer :: brom, broma, crom, croma, cromaa, pcrom
+double precision, dimension(:), pointer :: brom_eos, crom_eos
+double precision, dimension(:), allocatable, target :: cproa_rho_tc
 double precision, dimension(:), pointer :: coefa_k, coefb_k
 double precision, dimension(:), pointer :: coefa_p, coefb_p
 double precision, dimension(:,:), allocatable :: rij
@@ -250,12 +238,12 @@ double precision, dimension(:), pointer :: cvara_r11, cvara_r22, cvara_r33
 double precision, dimension(:), pointer :: cvara_r12, cvara_r23, cvara_r13
 double precision, dimension(:,:), pointer :: cvara_rij
 double precision, dimension(:), pointer :: viscl, visct, c_estim
-double precision, allocatable, dimension(:) :: surfbm
 double precision, dimension(:,:), pointer :: lapla, lagr_st_vel
-double precision, dimension(:), pointer :: cpro_tsrho
 double precision, dimension(:,:), pointer :: cpro_gradp
 double precision, dimension(:), pointer :: cpro_wgrec_s
 double precision, dimension(:,:), pointer :: cpro_wgrec_v
+double precision, dimension(:), pointer :: imasfl, bmasfl
+double precision, dimension(:), pointer :: imasfl_prev, bmasfl_prev
 
 type(var_cal_opt) :: vcopt_p, vcopt_u, vcopt
 
@@ -265,6 +253,52 @@ type(var_cal_opt) :: vcopt_p, vcopt_u, vcopt
 ! 1. Initialization
 !===============================================================================
 
+! Id of the mass flux
+call field_get_key_int(ivarfl(iu), kimasf, iflmas)
+call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
+
+! Pointers to the mass fluxes
+call field_get_val_s(iflmas, imasfl)
+call field_get_val_s(iflmab, bmasfl)
+
+! Pointers to properties
+! Density at time n+1,iteration iterns+1
+call field_get_val_s(icrom, crom_eos)
+call field_get_val_s(ibrom, brom_eos)
+
+! Density at time (n)
+if (irovar.eq.1) then
+  call field_get_val_prev_s(icrom, croma)
+  call field_get_val_prev_s(ibrom, broma)
+else
+  croma => crom_eos
+  broma => brom_eos
+endif
+
+! Density at time (n-1) if needed
+if ((idilat.gt.1.or.ivofmt.ge.0).and.irovar.eq.1) then
+  call field_get_val_prev2_s(icrom, cromaa)
+endif
+
+! Density for the unsteady term (at time n)
+! Compressible algorithm (mass equation is already solved)
+! or Low Mach compressible algos with mass flux prediction
+if (ippmod(icompf).ge.0.or.(idilat.gt.1.and.ipredfl.eq.1.and.irovar.eq.1)) then
+  pcrom => croma
+
+! VOF algorithm and Low Mach compressible Algos: density at time n-1
+else if ((idilat.gt.1.or.ivofmt.ge.0).and.irovar.eq.1) then
+  if (iterns.eq.1) then
+    pcrom => cromaa
+  else
+    pcrom => croma
+  endif
+
+! Deprecated algo or constant density
+else
+  pcrom => crom_eos
+endif
+
 ! Allocate temporary arrays
 allocate(smbr(3,ncelet))
 allocate(fimp(3,3,ncelet))
@@ -273,6 +307,44 @@ allocate(tsimp(3,3,ncelet))
 call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
 call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt_p)
 
+! map the density pointer:
+! 1/4(n-1) + 1/2(n) + 1/4(n+1)
+! here replaced by (n)
+crom => croma
+brom => broma
+
+! Interpolation of rho^n-1/2 (stored in pcrom)
+! Interpolation of the mass flux at (n+1/2)
+! NB: the mass flux (n+1) is overwritten because not used after.
+! The mass flux for (n->n+1) will be recomputed in resopv
+! FIXME irovar=1 and if dt varies, use theta(rho) = theta(u)*...
+if (vcopt_u%thetav .lt. 1.d0 .and. iappel.eq.1 .and. iterns.gt.1) then
+  allocate(cproa_rho_tc(ncelet))
+
+  ! Pointer to the previous mass fluxes
+  call field_get_val_prev_s(iflmas, imasfl_prev)
+  call field_get_val_prev_s(iflmab, bmasfl_prev)
+
+  if (irovar.eq.1) then
+    ! remap the density pointer: n-1/2
+    do iel = 1, ncelet
+      cproa_rho_tc(iel) = vcopt_u%thetav * croma(iel) + (1.d0 - vcopt_u%thetav) * cromaa(iel)
+    enddo
+    pcrom => cproa_rho_tc
+  endif
+
+  ! Inner mass flux interpolation: n-1/2->n+1/2
+  do ifac = 1, nfac
+    imasfl(ifac) = vcopt_u%thetav * imasfl(ifac) + (1.d0 - vcopt_u%thetav) * imasfl_prev(ifac)
+  enddo
+
+  ! Boundary mass flux interpolation: n-1/2->n+1/2
+  do ifac = 1, nfabor
+    bmasfl(ifac) = vcopt_u%thetav * bmasfl(ifac) + (1.d0 - vcopt_u%thetav) * bmasfl_prev(ifac)
+  enddo
+
+endif
+
 idftnp = vcopt_u%idften
 
 if (iand(idftnp, ANISOTROPIC_LEFT_DIFFUSION).ne.0) allocate(viscce(6,ncelet))
@@ -282,15 +354,6 @@ if (iescal(iespre).gt.0) then
   allocate(eswork(3,ncelet))
 endif
 
-! Reperage de rho au bord
-call field_get_val_s(ibrom, brom)
-! Reperage de rho courant (ie en cas d'extrapolation rho^n+1/2)
-call field_get_val_s(icrom, crom)
-! Reperage de rho^n en cas d'extrapolation
-if (iroext.gt.0.or.idilat.gt.1) then
-  call field_get_val_prev_s(icrom, croma)
-endif
-
 if (iappel.eq.2) then
   if (iforbr.ge.0 .and. iterns.eq.1 .or. ivofmt.ge.0) then
     call field_get_val_s(ivarfl(ipr), cvar_pr)
@@ -344,6 +407,71 @@ else
   c_st_vel => null()
 endif
 
+!-------------------------------------------------------------------------------
+! ---> Get user source terms
+
+do iel = 1, ncel
+  do isou = 1, 3
+    tsexp(isou,iel) = 0.d0
+    do jsou = 1, 3
+      tsimp(jsou,isou,iel) = 0.d0
+    enddo
+  enddo
+enddo
+
+! The computation of explicit and implicit source terms is performed
+! at the first iteration only.
+! If iphydr=1 or if we have buoyant scalars then we need to update source terms
+if (iihmpr.eq.1) then
+  call uitsnv (vel, tsexp, tsimp)
+endif
+
+call ustsnv &
+  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
+  iu   ,                                                         &
+  icepdc , icetsm , itypsm ,                                     &
+  dt     ,                                                       &
+  ckupdc , smacel , tsexp  , tsimp  )
+
+n_fans = cs_fan_n_fans()
+if (n_fans .gt. 0) then
+  if (ntcabs.eq.ntpabs+1) then
+    call debvtl(imasfl, bmasfl, crom, brom)
+  endif
+  call tsvvtl(tsexp)
+endif
+
+! Skip first time step after restart if previous values have not been read.
+if (vcopt_u%ibdtso.lt.0) then
+  vcopt_u%ibdtso = iabs(vcopt_u%ibdtso)
+  call field_set_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
+endif
+
+! Nudging towards optimal interpolation for velocity
+if (ippmod(iatmos).ge.0) then
+  call field_get_key_int(ivarfl(iu), kopint, f_oi_id)
+  if (f_oi_id.ge.0) then
+    call cs_at_data_assim_source_term(ivarfl(iu), tsexp, tsimp)
+  endif
+  if (iatmst.eq.1) then
+    call cs_at_source_term_for_inlet(tsexp)
+  endif
+endif
+
+! Coupling between two Code_Saturne
+if (nbrcpl.gt.0) then
+  !vectorial interleaved exchange
+  call csccel(iu, vela, coefav, coefbv, tsexp)
+endif
+
+if (vcopt_u%ibdtso.gt.1.and.ntcabs.gt.ntinit &
+  .and.(idtvar.eq.0.or.idtvar.eq.1)) then
+  ! TODO: remove test on ntcabs and implemente a "proper" condition for
+  ! initialization.
+  f_id = ivarfl(iu)
+  call cs_backward_differentiation_in_time(f_id, tsexp, tsimp)
+endif
+
 !===============================================================================
 ! 2. Potential forces (pressure gradient and gravity)
 !===============================================================================
@@ -356,7 +484,7 @@ if (f_id.ge.0) then
   call field_get_val_v(f_id, cpro_gradp)
 else
   allocate(grad(3,ncelet))
- cpro_gradp => grad
+  cpro_gradp => grad
 endif
 
 iccocg = 1
@@ -392,6 +520,8 @@ if (vcopt_p%iwgrec.eq.1) then
   endif
 endif
 
+call grdpor(inc)
+
 ! Pressure gradient
 call field_gradient_potential(ivarfl(ipr), iprev, imrgra, inc,    &
                               iccocg, iphydr,                     &
@@ -430,71 +560,45 @@ if (iforbr.ge.0 .and. iterns.eq.1) then
   enddo
 endif
 
-!     Au premier appel, TRAV est construit directement ici.
-!     Au second  appel (estimateurs), TRAV contient deja
-!       l'increment temporel.
-!     On pourrait fusionner en initialisant TRAV a zero
-!       avant le premier appel, mais ca fait des operations en plus.
-
-!     Remarques :
-!       rho g sera a l'ordre 2 s'il est extrapole.
-!       si on itere sur navsto, ca ne sert a rien de recalculer rho g a
-!         chaque fois (ie on pourrait le passer dans trava) mais ce n'est
-!         pas cher.
 if (iappel.eq.1) then
-  if (iphydr.eq.1) then
-    do iel = 1, ncel
-      trav(1,iel) = (frcxt(1 ,iel) - cpro_gradp(1,iel)) * cell_f_vol(iel)
-      trav(2,iel) = (frcxt(2 ,iel) - cpro_gradp(2,iel)) * cell_f_vol(iel)
-      trav(3,iel) = (frcxt(3 ,iel) - cpro_gradp(3,iel)) * cell_f_vol(iel)
-    enddo
-  else if (iphydr.eq.2) then
-    do iel = 1, ncel
-      rom = crom(iel)
-      trav(1,iel) = (rom*gx - grdphd(1,iel) - cpro_gradp(1,iel)) * cell_f_vol(iel)
-      trav(2,iel) = (rom*gy - grdphd(2,iel) - cpro_gradp(2,iel)) * cell_f_vol(iel)
-      trav(3,iel) = (rom*gz - grdphd(3,iel) - cpro_gradp(3,iel)) * cell_f_vol(iel)
-    enddo
-  else if (ippmod(icompf).ge.0) then
-    do iel = 1, ncel
-      rom = crom(iel)
-      trav(1,iel) = (rom*gx - cpro_gradp(1,iel)) * cell_f_vol(iel)
-      trav(2,iel) = (rom*gy - cpro_gradp(2,iel)) * cell_f_vol(iel)
-      trav(3,iel) = (rom*gz - cpro_gradp(3,iel)) * cell_f_vol(iel)
-    enddo
-  else
-    do iel = 1, ncel
-      drom = (crom(iel)-ro0)
-      trav(1,iel) = (drom*gx - cpro_gradp(1,iel) ) * cell_f_vol(iel)
-      trav(2,iel) = (drom*gy - cpro_gradp(2,iel) ) * cell_f_vol(iel)
-      trav(3,iel) = (drom*gz - cpro_gradp(3,iel) ) * cell_f_vol(iel)
-    enddo
-  endif
 
-else if(iappel.eq.2) then
-
-  if (iphydr.eq.1) then
-    do iel = 1, ncel
-      trav(1,iel) = trav(1,iel) + (frcxt(1 ,iel) - cpro_gradp(1,iel))*cell_f_vol(iel)
-      trav(2,iel) = trav(2,iel) + (frcxt(2 ,iel) - cpro_gradp(2,iel))*cell_f_vol(iel)
-      trav(3,iel) = trav(3,iel) + (frcxt(3 ,iel) - cpro_gradp(3,iel))*cell_f_vol(iel)
-    enddo
-  else if (iphydr.eq.2) then
-    do iel = 1, ncel
-      rom = crom(iel)
-      trav(1,iel) = trav(1,iel) + (rom*gx - grdphd(1,iel) - cpro_gradp(1,iel))*cell_f_vol(iel)
-      trav(2,iel) = trav(2,iel) + (rom*gy - grdphd(2,iel) - cpro_gradp(2,iel))*cell_f_vol(iel)
-      trav(3,iel) = trav(3,iel) + (rom*gz - grdphd(3,iel) - cpro_gradp(3,iel))*cell_f_vol(iel)
-    enddo
-  else
-    do iel = 1, ncel
-      drom = (crom(iel)-ro0)
-      trav(1,iel) = trav(1,iel) + (drom*gx - cpro_gradp(1,iel))*cell_f_vol(iel)
-      trav(2,iel) = trav(2,iel) + (drom*gy - cpro_gradp(2,iel))*cell_f_vol(iel)
-      trav(3,iel) = trav(3,iel) + (drom*gz - cpro_gradp(3,iel))*cell_f_vol(iel)
-    enddo
-  endif
+  ! Initialization
+  ! NB: at the second call, trav contains the temporal increment
+  do iel = 1, ncel
+    trav(1,iel) = 0.d0
+    trav(2,iel) = 0.d0
+    trav(3,iel) = 0.d0
+  enddo
+endif
 
+! FIXME : "rho g" will be second order only if extrapolated
+if (iphydr.eq.1) then
+  do iel = 1, ncel
+    trav(1,iel) = trav(1,iel)+(frcxt(1 ,iel) - cpro_gradp(1,iel)) * cell_f_vol(iel)
+    trav(2,iel) = trav(2,iel)+(frcxt(2 ,iel) - cpro_gradp(2,iel)) * cell_f_vol(iel)
+    trav(3,iel) = trav(3,iel)+(frcxt(3 ,iel) - cpro_gradp(3,iel)) * cell_f_vol(iel)
+  enddo
+else if (iphydr.eq.2) then
+  do iel = 1, ncel
+    rom = crom(iel)
+    trav(1,iel) = trav(1,iel)+(rom*gx - grdphd(1,iel) - cpro_gradp(1,iel)) * cell_f_vol(iel)
+    trav(2,iel) = trav(2,iel)+(rom*gy - grdphd(2,iel) - cpro_gradp(2,iel)) * cell_f_vol(iel)
+    trav(3,iel) = trav(3,iel)+(rom*gz - grdphd(3,iel) - cpro_gradp(3,iel)) * cell_f_vol(iel)
+  enddo
+else if (ippmod(icompf).ge.0) then
+  do iel = 1, ncel
+    rom = crom(iel)
+    trav(1,iel) = trav(1,iel)+(rom*gx - cpro_gradp(1,iel)) * cell_f_vol(iel)
+    trav(2,iel) = trav(2,iel)+(rom*gy - cpro_gradp(2,iel)) * cell_f_vol(iel)
+    trav(3,iel) = trav(3,iel)+(rom*gz - cpro_gradp(3,iel)) * cell_f_vol(iel)
+  enddo
+else
+  do iel = 1, ncel
+    drom = (crom(iel)-ro0)
+    trav(1,iel) = trav(1,iel)+(drom*gx - cpro_gradp(1,iel) ) * cell_f_vol(iel)
+    trav(2,iel) = trav(2,iel)+(drom*gy - cpro_gradp(2,iel) ) * cell_f_vol(iel)
+    trav(3,iel) = trav(3,iel)+(drom*gz - cpro_gradp(3,iel) ) * cell_f_vol(iel)
+  enddo
 endif
 
 ! Free memory
@@ -513,18 +617,16 @@ if (allocated(grad)) deallocate(grad)
 
 
 !-------------------------------------------------------------------------------
-! ---> INITIALISATION DU TABLEAU TRAVA et terme source AU PREMIER PASSAGE
-!     (A LA PREMIERE ITER SUR NAVSTO)
+! ---> Initialize trava array and source terms at the first call (iterns=1)
 
-!     TRAVA rassemble les termes sources qu'il suffit de calculer
-!       a la premiere iteration sur navsto quand il y a plusieurs iter.
-!     Quand il n'y a qu'une iter, on cumule directement dans TRAV
-!       ce qui serait autrement alle dans TRAVA
-!     Les termes sources explicites serviront
-!       pour le pas de temps suivant en cas d'extrapolation (plusieurs
-!       iter sur navsto ou pas)
+!     trava contains all source terms needed from the first sub iteration
+!       (iterns=1) for the other iterations.
+!     When there is only one iteration, we build source terms directly in trav
+!       array.
+!     Explicit source terms will be used at the next time step in case of
+!       extrapolation (if there is only one or many iteration on navtsv)
 
-!     A la premiere iter sur navsto
+! At the first iteration on navstv
 if (iterns.eq.1) then
 
   ! Si on   extrapole     les T.S. : -theta*valeur precedente
@@ -571,25 +673,11 @@ endif
 
 if (iappel.eq.1) then
 
-  ! Low Mach compressible Algos
-  if (idilat.gt.1.or.ippmod(icompf).ge.0) then
-    call field_get_val_prev_s(icrom, pcrom)
-
-  ! VOF algorithm
-  else if (ivofmt.ge.0) then
-    call field_get_val_s(icroaa, pcrom)
-
-  ! Standard algo
-  else
-
-    call field_get_val_s(icrom, pcrom)
-  endif
-
   do iel = 1, ncel
     do isou = 1, 3
       fimp(isou,isou,iel) = vcopt_u%istat*pcrom(iel)/dt(iel)*cell_f_vol(iel)
       do jsou = 1, 3
-        if(jsou.ne.isou) fimp(isou,jsou,iel) = 0.d0
+        if(jsou.ne.isou) fimp(jsou,isou,iel) = 0.d0
       enddo
     enddo
   enddo
@@ -600,7 +688,7 @@ else
   do iel = 1, ncel
     do isou = 1, 3
       do jsou = 1, 3
-        fimp(isou,jsou,iel) = 0.d0
+        fimp(jsou,isou,iel) = 0.d0
       enddo
     enddo
   enddo
@@ -639,8 +727,6 @@ if(     (itytur.eq.2 .or. itytur.eq.5 .or. iturb.eq.60) &
     ! Calcul de rho^n grad k^n      si rho non extrapole
     !           rho^n grad k^n      si rho     extrapole
 
-    call field_get_val_s(icrom, crom)
-    call field_get_val_prev_s(icrom, croma)
     do iel = 1, ncel
       romvom = -croma(iel)*cell_f_vol(iel)*d2s3
       do isou = 1, 3
@@ -711,40 +797,30 @@ endif
 if ((ncepdp.gt.0).and.(iphydr.ne.1)) then
 
   ! Les termes diagonaux sont places dans TRAV ou TRAVA,
-  !   La prise en compte de uvwk a partir de la seconde iteration
+  !   La prise en compte de velk a partir de la seconde iteration
   !   est faite directement dans coditv.
   if (iterns.eq.1) then
 
-    ! On utilise temporairement TRAV comme tableau de travail.
-    ! Son contenu est stocke dans W7, W8 et W9 jusqu'apres tspdcv
-    do iel = 1,ncel
-      w7(iel) = trav(1,iel)
-      w8(iel) = trav(2,iel)
-      w9(iel) = trav(3,iel)
-      trav(1,iel) = 0.d0
-      trav(2,iel) = 0.d0
-      trav(3,iel) = 0.d0
-    enddo
+    allocate(hl_exp(3, ncelet))
 
-    call tspdcv(ncepdp, icepdc, vela, ckupdc, trav)
+    call tspdcv(ncepdp, icepdc, vela, ckupdc, hl_exp)
 
-    ! Si on itere sur navsto, on utilise TRAVA ; sinon TRAV
+    ! If PISO-like sub-iterations, we use trava, otherwise trav
     if(nterup.gt.1) then
       do iel = 1, ncel
-        trava(1,iel) = trava(1,iel) + trav(1,iel)
-        trava(2,iel) = trava(2,iel) + trav(2,iel)
-        trava(3,iel) = trava(3,iel) + trav(3,iel)
-        trav(1,iel)  = w7(iel)
-        trav(2,iel)  = w8(iel)
-        trav(3,iel)  = w9(iel)
+        trava(1,iel) = trava(1,iel) + hl_exp(1,iel)
+        trava(2,iel) = trava(2,iel) + hl_exp(2,iel)
+        trava(3,iel) = trava(3,iel) + hl_exp(3,iel)
       enddo
     else
       do iel = 1, ncel
-        trav(1,iel)  = w7(iel) + trav(1,iel)
-        trav(2,iel)  = w8(iel) + trav(2,iel)
-        trav(3,iel)  = w9(iel) + trav(3,iel)
+        trav(1,iel) = trav(1,iel) + hl_exp(1,iel)
+        trav(2,iel) = trav(2,iel) + hl_exp(2,iel)
+        trav(3,iel) = trav(3,iel) + hl_exp(3,iel)
       enddo
     endif
+
+    deallocate(hl_exp)
   endif
 
 endif
@@ -762,12 +838,12 @@ if (iappel.eq.1) then
 
       ! Diagonal part
       do isou = 1, 3
-        fimp(isou,isou,iel) = fimp(isou,isou,iel) + romvom*ckupdc(ielpdc,isou)
+        fimp(isou,isou,iel) = fimp(isou,isou,iel) + romvom*ckupdc(isou,ielpdc)
       enddo
       ! Extra-diagonal part
-      cpdc12 = ckupdc(ielpdc,4)
-      cpdc23 = ckupdc(ielpdc,5)
-      cpdc13 = ckupdc(ielpdc,6)
+      cpdc12 = ckupdc(4,ielpdc)
+      cpdc23 = ckupdc(5,ielpdc)
+      cpdc13 = ckupdc(6,ielpdc)
 
       fimp(1,2,iel) = fimp(1,2,iel) + romvom*cpdc12
       fimp(2,1,iel) = fimp(2,1,iel) + romvom*cpdc12
@@ -1117,12 +1193,12 @@ endif
 !      into account (only for the first call, the second one is dedicated
 !      to error estimators)
 
-if (iappel.eq.1.and.iphydr.eq.1.and.iterns.eq.1) then
+if (iappel.eq.1.and.iphydr.eq.1) then
 
-! force ext au pas de temps precedent :
-!     FRCXT a ete initialise a zero
-!     (est deja utilise dans typecl, et est mis a jour a la fin
-!     de navsto)
+  ! External forces at previous time step:
+  !     frcxt was initialised to 0
+  !     NB: frcxt was used in typecl, and will be updated
+  !         at the end of navstv
 
   do iel = 1, ncel
 
@@ -1141,12 +1217,12 @@ if (iappel.eq.1.and.iphydr.eq.1.and.iterns.eq.1) then
       vit1   = vela(1,iel)
       vit2   = vela(2,iel)
       vit3   = vela(3,iel)
-      cpdc11 = ckupdc(ielpdc,1)
-      cpdc22 = ckupdc(ielpdc,2)
-      cpdc33 = ckupdc(ielpdc,3)
-      cpdc12 = ckupdc(ielpdc,4)
-      cpdc23 = ckupdc(ielpdc,5)
-      cpdc13 = ckupdc(ielpdc,6)
+      cpdc11 = ckupdc(1,ielpdc)
+      cpdc22 = ckupdc(2,ielpdc)
+      cpdc33 = ckupdc(3,ielpdc)
+      cpdc12 = ckupdc(4,ielpdc)
+      cpdc23 = ckupdc(5,ielpdc)
+      cpdc13 = ckupdc(6,ielpdc)
       dfrcxt(1 ,iel) = dfrcxt(1 ,iel) &
                     - crom(iel)*(cpdc11*vit1+cpdc12*vit2+cpdc13*vit3)
       dfrcxt(2 ,iel) = dfrcxt(2 ,iel) &
@@ -1176,13 +1252,30 @@ if (iappel.eq.1.and.iphydr.eq.1.and.iterns.eq.1) then
   ! Add -div( rho R) as external force
   if (itytur.eq.3.and.igprij.eq.1) then
     do iel = 1, ncel
-      dvol = 1.d0/cell_f_vol(iel)
+      dvol = 0.d0
+      ! If it is not a solid cell
+      if (isolid(iporos, iel).eq.0) dvol = 1.d0 / cell_f_vol(iel)
       do isou = 1, 3
         dfrcxt(isou, iel) = dfrcxt(isou, iel) - divt(isou, iel)*dvol
       enddo
     enddo
   endif
 
+  ! ---> Use user source terms
+
+  if (igpust.eq.1) then
+    do iel = 1, ncel
+      dvol = 0.d0
+      ! If it is not a solid cell
+      if (isolid(iporos, iel).eq.0) dvol = 1.d0 / cell_f_vol(iel)
+
+      do isou = 1, 3
+        dfrcxt(isou, iel) = dfrcxt(isou, iel) + tsexp(isou, iel)*dvol
+      enddo
+    enddo
+
+  endif
+
   if (irangp.ge.0.or.iperio.eq.1) then
     call synvin(dfrcxt)
   endif
@@ -1219,96 +1312,7 @@ if (iappel.eq.2) then
 endif
 
 !-------------------------------------------------------------------------------
-! ---> User source terms
-
-do iel = 1, ncel
-  do isou = 1, 3
-    tsexp(isou,iel) = 0.d0
-    do jsou = 1, 3
-      tsimp(isou,jsou,iel) = 0.d0
-    enddo
-  enddo
-enddo
-
-! The computation of explicit and implicit source terms is performed
-! at the first iteration only.
-if (iterns.eq.1) then
-
-  if (iihmpr.eq.1) then
-    call uitsnv (vel, tsexp, tsimp)
-  endif
-
-  n_fans = cs_fan_n_fans()
-  if (n_fans .gt. 0) then
-    if (ntcabs.eq.ntpabs+1) then
-      call debvtl(flumas, flumab, crom, brom)
-    endif
-    call tsvvtl(tsexp)
-  endif
-
-  call ustsnv &
- ( nvar   , nscal  , ncepdp , ncesmp ,                            &
-   iu   ,                                                         &
-   icepdc , icetsm , itypsm ,                                     &
-   dt     ,                                                       &
-   ckupdc , smacel , tsexp  , tsimp  )
-
-  if (vcopt_u%ibdtso.gt.1.and.ntcabs.gt.ntinit &
-      .and.(idtvar.eq.0.or.idtvar.eq.1)) then
-    ! TODO: remove test on ntcabs and implemente a "proper" condition for
-    ! initialization.
-    f_id = ivarfl(iu)
-    call cs_backward_differentiation_in_time(f_id, tsexp, tsimp)
-  endif
-  ! Skip first time step after restart if previous values have not been read.
-  if (vcopt_u%ibdtso.lt.0) then
-    vcopt_u%ibdtso = iabs(vcopt_u%ibdtso)
-    call field_set_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
-  endif
-
-  ! Nudging towards optimal interpolation for velocity
-  if (ippmod(iatmos).ge.0) then
-    call field_get_key_int(ivarfl(iu), kopint, f_oi_id)
-    if (f_oi_id.ge.0) then
-      call cs_at_data_assim_source_term(ivarfl(iu), tsexp, tsimp)
-    endif
-    if (iatmst.eq.1) then
-      call cs_at_source_term_for_inlet(tsexp)
-    endif
-  endif
-
-  ! Coupling between two Code_Saturne
-  if (nbrcpl.gt.0) then
-    !vectorial interleaved exchange
-    call csccel(iu, vela, coefav, coefbv, tsexp)
-  endif
-
-  if (iphydr.eq.1.and.igpust.eq.1) then
-    do iel = 1, ncel
-      !FIXME when using porosity
-      dvol = 1.d0/cell_f_vol(iel)
-      do isou = 1, 3
-        dfrcxt(isou, iel) = dfrcxt(isou, iel) + tsexp(isou, iel)*dvol
-      enddo
-    enddo
-
-    if (irangp.ge.0.or.iperio.eq.1) then
-      call synvin(dfrcxt)
-    endif
-  endif
-
-endif
-
-! if PISO sweeps are expected, implicit user sources terms are stored in ximpa
-if (iterns.eq.1.and.nterup.gt.1) then
-  do iel = 1, ncel
-    do isou = 1, 3
-      do jsou = 1, 3
-        ximpa(isou,jsou,iel) = tsimp(isou,jsou,iel)
-      enddo
-    enddo
-  enddo
-endif
+! ---> Use user source terms
 
 ! ---> Explicit contribution due to implicit terms
 
@@ -1368,45 +1372,23 @@ if (iappel.eq.1) then
   ! If source terms are time-extrapolated
   if (isno2t.gt.0) then
     thetap = vcopt_u%thetav
-    if (iterns.gt.1) then
-      do iel = 1, ncel
-        do isou = 1, 3
-          do jsou = 1, 3
-            fimp(isou,jsou,iel) = fimp(isou,jsou,iel)                      &
-                                - ximpa(isou,jsou,iel)*thetap
-          enddo
+    do iel = 1, ncel
+      do isou = 1, 3
+        do jsou = 1, 3
+          fimp(jsou,isou,iel) = fimp(jsou,isou,iel)                      &
+                              - tsimp(jsou,isou,iel)*thetap
         enddo
       enddo
-    else
-      do iel = 1, ncel
-        do isou = 1, 3
-          do jsou = 1, 3
-            fimp(isou,jsou,iel) = fimp(isou,jsou,iel)                      &
-                                - tsimp(isou,jsou,iel)*thetap
-          enddo
-        enddo
-      enddo
-    endif
+    enddo
   else
-    if (iterns.gt.1) then
-      do iel = 1, ncel
-        do isou = 1, 3
-          do jsou = 1, 3
-            fimp(isou,jsou,iel) = fimp(isou,jsou,iel)                      &
-                                + max(-ximpa(isou,jsou,iel),zero)
-          enddo
+    do iel = 1, ncel
+      do isou = 1, 3
+        do jsou = 1, 3
+          fimp(jsou,isou,iel) = fimp(jsou,isou,iel)                      &
+                              + max(-tsimp(jsou,isou,iel),zero)
         enddo
       enddo
-    else
-      do iel = 1, ncel
-        do isou = 1, 3
-          do jsou = 1, 3
-            fimp(isou,jsou,iel) = fimp(isou,jsou,iel)                      &
-                                + max(-tsimp(isou,jsou,iel),zero)
-          enddo
-        enddo
-      enddo
-    endif
+    enddo
   endif
 endif
 
@@ -1580,40 +1562,18 @@ if (iappel.eq.1) then
 
   iescap = iescal(iespre)
 
-  if (iterns.eq.1) then
-
-    ! Warning: in case of convergence estimators, eswork give the estimator
-    ! of the predicted velocity
-    call coditv &
- ( idtvar , iterns , ivarfl(iu)      , iconvp , idiffp , ndircp ,&
-   imrgra , nswrsp , nswrgp , imligp , ircflp , ivisse ,          &
-   ischcp , isstpp , iescap , idftnp , iswdyp ,                   &
-   iwarnp ,                                                       &
-   blencp , epsilp , epsrsp , epsrgp , climgp ,                   &
-   relaxp , thetap ,                                              &
-   vela   , vela   ,                                              &
-   coefav , coefbv , cofafv , cofbfv ,                            &
-   flumas , flumab ,                                              &
-   viscfi , viscbi , viscf  , viscb  , secvif , secvib ,          &
-   rvoid  , rvoid  , rvoid  ,                                     &
-   icvflb , icvfli ,                                              &
-   fimp   ,                                                       &
-   smbr   ,                                                       &
-   vel    ,                                                       &
-   eswork )
-
-  else if(iterns.gt.1) then
-
-    call coditv &
+  ! Warning: in case of convergence estimators, eswork give the estimator
+  ! of the predicted velocity
+  call coditv &
  ( idtvar , iterns , ivarfl(iu)      , iconvp , idiffp , ndircp , &
    imrgra , nswrsp , nswrgp , imligp , ircflp , ivisse ,          &
    ischcp , isstpp , iescap , idftnp , iswdyp ,                   &
    iwarnp ,                                                       &
    blencp , epsilp , epsrsp , epsrgp , climgp ,                   &
    relaxp , thetap ,                                              &
-   vela   , uvwk   ,                                              &
+   vela   , velk   ,                                              &
    coefav , coefbv , cofafv , cofbfv ,                            &
-   flumas , flumab ,                                              &
+   imasfl , bmasfl ,                                              &
    viscfi , viscbi , viscf  , viscb  , secvif , secvib ,          &
    rvoid  , rvoid  , rvoid  ,                                     &
    icvflb , icvfli ,                                              &
@@ -1622,8 +1582,6 @@ if (iappel.eq.1) then
    vel    ,                                                       &
    eswork )
 
-  endif
-
   ! Velocity-pression coupling: compute the vector T, stored in tpucou,
   !  coditv is called, only one sweep is done, and tpucou is initialized
   !  by 0. so that the advection/diffusion added by bilscv is 0.
@@ -1659,7 +1617,7 @@ if (iappel.eq.1) then
    relaxp , thetap ,                                              &
    vect   , vect   ,                                              &
    coefav , coefbv , cofafv , cofbfv ,                            &
-   flumas , flumab ,                                              &
+   imasfl , bmasfl ,                                              &
    viscfi , viscbi , viscf  , viscb  , secvif , secvib ,          &
    rvoid  , rvoid  , rvoid  ,                                     &
    icvflb , ivoid  ,                                              &
@@ -1710,7 +1668,7 @@ else if (iappel.eq.2) then
    blencp , epsrgp , climgp , relaxp , thetap ,                            &
    vel    , vel    ,                                                       &
    coefav , coefbv , cofafv , cofbfv ,                                     &
-   flumas , flumab , viscf  , viscb  , secvif , secvib ,                   &
+   imasfl , bmasfl , viscf  , viscb  , secvif , secvib ,                   &
    rvoid  , rvoid  , rvoid  ,                                              &
    icvflb , icvfli ,                                                       &
    smbr   )
@@ -1790,6 +1748,7 @@ deallocate(tsexp)
 deallocate(tsimp)
 if (allocated(viscce)) deallocate(viscce)
 if (allocated(divt)) deallocate(divt)
+if (allocated(cproa_rho_tc)) deallocate(cproa_rho_tc)
 
 !--------
 ! Formats
