@@ -109,6 +109,7 @@ integer          iok   , nfld  , f_id  , f_dim  , f_type
 integer          nbccou
 integer          ntrela
 integer          icmst
+integer          st_id
 
 integer          isvhb, iz
 integer          ii
@@ -129,9 +130,6 @@ integer          ipass
 data             ipass /0/
 save             ipass
 
-integer          infpar
-save             infpar
-
 integer, allocatable, dimension(:,:) :: icodcl
 integer, allocatable, dimension(:) :: ilzfbr
 integer, allocatable, dimension(:) :: isostd
@@ -145,11 +143,9 @@ double precision, allocatable, dimension(:) :: hbord, theipb
 double precision, allocatable, dimension(:) :: visvdr
 double precision, allocatable, dimension(:) :: prdv2f
 double precision, allocatable, dimension(:) :: mass_source
-double precision, dimension(:), pointer :: brom, crom, crom_prev, crom_prev2
+double precision, dimension(:), pointer :: brom, crom
 
-double precision, pointer, dimension(:,:) :: uvwk
 double precision, pointer, dimension(:,:) :: trava
-double precision, pointer, dimension(:,:,:) :: ximpav
 double precision, dimension(:,:), pointer :: disale
 double precision, dimension(:,:), pointer :: vel
 double precision, dimension(:,:), pointer :: cvar_vec
@@ -187,7 +183,7 @@ interface
     isostd ,                                                       &
     dt     ,                                                       &
     frcxt  ,                                                       &
-    trava  , ximpa  , uvwk   )
+    trava  )
 
     use dimens, only: ndimfb
     use mesh, only: nfabor
@@ -200,8 +196,7 @@ interface
 
     double precision, pointer, dimension(:)   :: dt
     double precision, pointer, dimension(:,:) :: frcxt
-    double precision, pointer, dimension(:,:) :: trava, uvwk
-    double precision, pointer, dimension(:,:,:) :: ximpa
+    double precision, pointer, dimension(:,:) :: trava
 
   end subroutine navstv
 
@@ -233,7 +228,7 @@ call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
 call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt_p)
 
 !===============================================================================
-! 1.  INITIALISATION
+! 1. Initialisation
 !===============================================================================
 
 allocate(isostd(nfabor+1))
@@ -412,16 +407,13 @@ if (ipass.eq.1) then
 endif
 
 !===============================================================================
-! 5. DANS LE CAS  "zero pas de temps" EN "SUITE" DE CALCUL
-!      ON SORT ICI
+! 5. Temporal update of previous values (mass flux, density, ...)
 !===============================================================================
 !  on sort avant SCHTMP car sinon a l'ordre 2 en temps la valeur du
 !  flux de masse au pas de temps precedent est ecrasee par la valeur
-!  au pas de temps actuel et la valeur au pas de temps actuel est
-!  remplacee par une extrapolation qui n'a pas lieu d'etre puisque
-!  NTCABS n'est pas incremente. Dans le cas INPDT0=1 sans suite, il
+!  au pas de temps actuel. Dans le cas INPDT0=1 sans suite, il
 !  n'y a pas de probleme puisque tous les flux de masse sont a 0           !
-!  Si ITRALE=0, on est a l'iteration d'initialisation de l'ALE,
+!  Si itrale=0, on est a l'iteration d'initialisation de l'ALE,
 !  on ne touche pas au flux de masse non plus
 
 
@@ -432,32 +424,6 @@ if (itrale.gt.0) then
   call schtmp(nscal, iappel)
 endif
 
-!===============================================================================
-! 5.bis Current to previous for density
-!===============================================================================
-
-! --- Noter que exceptionnellement, on fait un calcul avec ncelet,
-!     pour eviter une nouvelle communication
-
-! If required, the density at time step n-2 is updated
-if (ivofmt.ge.0.or.idilat.gt.1) then
-  call field_get_val_prev_s(icrom, crom_prev)
-  call field_get_val_s(icroaa, crom_prev2)
-  do iel = 1, ncelet
-    crom_prev2(iel) = crom_prev(iel)
-  enddo
-endif
-
-! If required, the density at time step n-1 is updated
-if (icalhy.eq.1.or.idilat.gt.1.or.ivofmt.ge.0.or.ipthrm.eq.1) then
-  call field_get_val_s(icrom, crom)
-  call field_get_val_prev_s(icrom, crom_prev)
-  do iel = 1, ncelet
-    crom_prev(iel) = crom(iel)
-  enddo
-  call field_current_to_previous(ibrom)
-endif
-
 !===============================================================================
 ! 6.  MISE A JOUR DE LA LOCALISATION DES INTERFACES DE COUPLAGE CS/CS
 !===============================================================================
@@ -496,19 +462,10 @@ endif
 !    UN UTILISATEUR DECIDERAIT D'AVOIR UN COEFF DE PDC DEPENDANT DE
 !    LA VITESSE MOYENNE OU MAX.
 
-
 if (ncpdct.gt.0) then
 
   call cs_head_losses_compute(ckupdc)
 
-  iappel = 3
-
-  call cs_f_user_head_losses &
-( ncepdc , iappel ,                                              &
-  icepdc , izcpdc ,                                              &
-  dt     ,                                                       &
-  ckupdc )
-
  if (iflow .eq.1) then
    call laghlo(ncepdc, icepdc, itypfb, ckupdc)
  endif
@@ -664,9 +621,17 @@ do f_id = 0, nfld - 1
   ! Is the field of type FIELD_VARIABLE?
   if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
     call field_current_to_previous(f_id)
+
+    ! For buoyant scalar with source termes, current to previous for them
+    call field_get_key_int(f_id, kst, st_id)
+    if (st_id .ge.0) then
+      call field_current_to_previous(st_id)
+    endif
   endif
 enddo
 
+
+
 if (ippmod(idarcy).eq.1) then
 
   ! Index of the corresponding field
@@ -734,12 +699,12 @@ if (iand(vcopt_p%idften, ANISOTROPIC_DIFFUSION).ne.0               &
     iel = icepdc(ielpdc)
 
     ! dttens = (1/dt + Kpdc)^-1
-    hdls(1) = ckupdc(ielpdc, 1) + 1.d0/dt(iel)
-    hdls(2) = ckupdc(ielpdc, 2) + 1.d0/dt(iel)
-    hdls(3) = ckupdc(ielpdc, 3) + 1.d0/dt(iel)
-    hdls(4) = ckupdc(ielpdc, 4)
-    hdls(5) = ckupdc(ielpdc, 5)
-    hdls(6) = ckupdc(ielpdc, 6)
+    hdls(1) = ckupdc(1, ielpdc) + 1.d0/dt(iel)
+    hdls(2) = ckupdc(2, ielpdc) + 1.d0/dt(iel)
+    hdls(3) = ckupdc(3, ielpdc) + 1.d0/dt(iel)
+    hdls(4) = ckupdc(4, ielpdc)
+    hdls(5) = ckupdc(5, ielpdc)
+    hdls(6) = ckupdc(6, ielpdc)
 
     call symmetric_matrix_inverse(hdls, dttens(:, iel))
   enddo
@@ -832,12 +797,8 @@ endif
 itrfup = 1
 
 if (nterup.gt.1) then
-  allocate(ximpav(ndim,ndim,ncelet))
-  allocate(uvwk(ndim,ncelet))
   allocate(trava(ndim,ncelet))
 else
-  ximpav => rvoid3
-  uvwk => rvoid2
   trava => rvoid2
 endif
 
@@ -892,7 +853,6 @@ do while (iterns.le.nterup)
     iqimp,  icalke, ientat, ientcp, inmoxy, ientox,                &
     ientfu, ientgb, ientgf, iprofm, iautom,                        &
     itypfb, izfppp, icodcl,                                        &
-    surfbo, cdgfbo,                                                &
     qimp,   qimpat, qimpcp, dh,     xintur,                        &
     timpat, timpcp, tkent ,  fment, distch, nvar, rcodcl)
 
@@ -1202,38 +1162,9 @@ do while (iterns.le.nterup)
 
 
   ! Compute y+ if needed
-
+  ! and Van Driest "amortissement"
   if (itytur.eq.4 .and. idries.eq.1) then
-    ! Compute the number of walls
-    if (ipass.eq.1) then
-      infpar = 0
-      do ifac = 1, nfabor
-        if (itypfb(ifac).eq.iparoi .or. itypfb(ifac).eq.iparug) then
-          infpar = infpar+1
-        endif
-      enddo
-      if (irangp.ge.0) then
-        call parcpt(infpar)
-      endif
-    endif
-
-    if (infpar.gt.0) then
-      call distyp(itypfb, yplpar)
-    else
-      do iel = 1, ncelet
-        yplpar(iel) = grand
-      enddo
-    endif
-
-  endif
-
-  if (itytur.eq.4 .and. idries.eq.1) then
-
-    ! Pas d'amortissement si pas de paroi
-    if (infpar.gt.0) then
-      call vandri(visvdr, yplpar)
-    endif
-
+    call distyp(itypfb, visvdr)
   endif
 
 !===============================================================================
@@ -1254,7 +1185,7 @@ do while (iterns.le.nterup)
     ! otherwise it is done in navstv.f90
     if (itrale.eq.0) then
 
-      call alelav(iterns)
+      call cs_ale_solve_mesh_velocity(iterns, ndircl(iuma), impale, ialtyb)
       must_return = .true.
 
     endif
@@ -1272,7 +1203,7 @@ do while (iterns.le.nterup)
     if (allocated(visvdr)) deallocate(visvdr)
 
     if (nterup.gt.1) then
-      deallocate(ximpav, uvwk, trava)
+      deallocate(trava)
     endif
 
     deallocate(icodcl, rcodcl)
@@ -1301,7 +1232,21 @@ do while (iterns.le.nterup)
     ! In case of buoyancy, scalars and momentum are coupled
     if (n_buoyant_scal.ge.1) then
 
+      if(vcopt_u%iwarni.ge.1) then
+        write(nfecra,1060)
+      endif
+
+      ! Update buoyant scalar(s)
+      call scalai(nvar, nscal , iterns , dt)
+
+      ! Diffusion terms for weakly compressible algorithm
+      if (idilat.ge.4) then
+        call diffst(nscal, iterns)
+      endif
+
       ! Update the density
+      !-------------------
+
       call phyvar(nvar, nscal, iterns, dt)
 
     endif
@@ -1319,7 +1264,7 @@ do while (iterns.le.nterup)
         isostd ,                                                       &
         dt     ,                                                       &
         frcxt  ,                                                       &
-        trava  , ximpav , uvwk   )
+        trava  )
 
       ! Update local pointer arrays for transient turbomachinery computations
       if (iturbo.eq.2) then
@@ -1364,23 +1309,6 @@ do while (iterns.le.nterup)
 
     endif
 
-    ! In case of buoyancy, scalars and momentum are coupled
-    if (n_buoyant_scal.ge.1) then
-
-      if(vcopt_u%iwarni.ge.1) then
-        write(nfecra,1060)
-      endif
-
-      ! Update buoyant scalar(s)
-      call scalai(nvar, nscal , iterns , dt)
-
-      ! Diffusion terms for weakly compressible algorithm
-      if (idilat.ge.4) then
-        call diffst(nscal, iterns)
-      endif
-
-    endif
-
     !     Si c'est la derniere iteration : INSLST = 1
     if ((icvrge.eq.1).or.(iterns.eq.nterup)) then
 
@@ -1399,8 +1327,8 @@ do while (iterns.le.nterup)
         inslst = 1
       endif
 
-      !     On teste le flux de masse
-      if ((istmpf.eq.0.and.inslst.eq.0) .or. istmpf.ne.0) then
+      ! For explicit mass flux
+      if (istmpf.eq.0.and.inslst.eq.0) then
         iappel = 3
         call schtmp(nscal, iappel)
       endif
@@ -1442,7 +1370,6 @@ if (ippmod(idarcy).eq.1) then
     iqimp,  icalke, ientat, ientcp, inmoxy, ientox,                &
     ientfu, ientgb, ientgf, iprofm, iautom,                        &
     itypfb, izfppp, icodcl,                                        &
-    surfbo, cdgfbo,                                                &
     qimp,   qimpat, qimpcp, dh,     xintur,                        &
     timpat, timpcp, tkent ,  fment, distch, nvar, rcodcl)
 
@@ -1496,7 +1423,7 @@ if (allocated(theipb)) deallocate(theipb)
 if (allocated(visvdr)) deallocate(visvdr)
 
 if (nterup.gt.1) then
-  deallocate(ximpav, uvwk, trava)
+  deallocate(trava)
 endif
 
 ! Calcul sur champ de vitesse fige SUITE (a cause de la boucle U/P)
@@ -1530,8 +1457,6 @@ if (iccvfg.eq.0) then
   endif
 
   !     On ne passe dans SCHTMP que si ISTMPF.EQ.0 (explicite)
-  !     On teste le flux de masse
-  !     pour conserver
   if (istmpf.eq.0) then
     iappel = 4
     call schtmp(nscal, iappel)
