@@ -20,35 +20,34 @@
 
 !-------------------------------------------------------------------------------
 
+!===============================================================================
+! Function:
+! ---------
+
+!> \file tspdcv.f90
+!>
+!> \brief This subroutine computes the explicit contribution of
+!> headlosses terms.
+!-------------------------------------------------------------------------------
+
+!-------------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]     ncepdp        number of cells with head loss
+!> \param[in]     icepdc        index of cells with head loss
+!> \param[in]     vela          velocity at the previous time step
+!> \param[in]     ckupdc        work array for the head loss
+!> \param[out]    trav          right hand side
+!_______________________________________________________________________________
+
+
 subroutine tspdcv &
  ( ncepdp , icepdc ,                                              &
    vela   ,                                                       &
    ckupdc , trav   )
 
-!===============================================================================
-! FONCTION :
-! ----------
-
-! CALCUL DES TERMES DE PERTE DE CHARGE
-!        POUR LE BILAN EXPLICITE
-!-------------------------------------------------------------------------------
-!ARGU                             ARGUMENTS
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! ncepdp           ! i  ! <-- ! number of cells with head loss                 !
-! icepdc(ncelet    ! te ! <-- ! numero des ncepdp cellules avec pdc            !
-! ckupdc           ! tr ! <-- ! tableau de travail pour pdc                    !
-!  (ncepdp,6)      !    !     !                                                !
-! trav(ncelet,3    ! tr ! <-- ! tableau des second membres                     !
-!__________________!____!_____!________________________________________________!
-
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
-!===============================================================================
-
 !===============================================================================
 ! Module files
 !===============================================================================
@@ -67,22 +66,27 @@ implicit none
 integer          ncepdp
 integer          icepdc(ncepdp)
 
-double precision ckupdc(ncepdp,6)
+double precision ckupdc(6,ncepdp)
 double precision trav(3,ncelet)
 double precision vela  (3  ,ncelet)
 
 ! Local variables
 
 integer          iel   , ielpdc
+integer          key_t_ext_id
+integer          iroext
 double precision romvom, vit1  , vit2  , vit3
 double precision cpdc11, cpdc22, cpdc33, cpdc12, cpdc13, cpdc23
 double precision, dimension(:), pointer :: crom, croma
 
 !===============================================================================
 
+! Time extrapolation?
+call field_get_key_id("time_extrapolated", key_t_ext_id)
 
 call field_get_val_s(icrom, crom)
 
+call field_get_key_int(icrom, key_t_ext_id, iroext)
 if (iroext.gt.0.and.isno2t.gt.0) then
   call field_get_val_prev_s(icrom, croma)
 endif
@@ -93,19 +97,19 @@ do ielpdc = 1, ncepdp
 
   iel    = icepdc(ielpdc)
   romvom =-crom(iel)*cell_f_vol(iel)
-  cpdc11 = ckupdc(ielpdc,1)
-  cpdc22 = ckupdc(ielpdc,2)
-  cpdc33 = ckupdc(ielpdc,3)
-  cpdc12 = ckupdc(ielpdc,4)
-  cpdc23 = ckupdc(ielpdc,5)
-  cpdc13 = ckupdc(ielpdc,6)
+  cpdc11 = ckupdc(1,ielpdc)
+  cpdc22 = ckupdc(2,ielpdc)
+  cpdc33 = ckupdc(3,ielpdc)
+  cpdc12 = ckupdc(4,ielpdc)
+  cpdc23 = ckupdc(5,ielpdc)
+  cpdc13 = ckupdc(6,ielpdc)
   vit1   = vela(1,iel)
   vit2   = vela(2,iel)
   vit3   = vela(3,iel)
 
-  trav(1,iel) = trav(1,iel) + romvom*(cpdc11*vit1 + cpdc12*vit2 + cpdc13*vit3)
-  trav(2,iel) = trav(2,iel) + romvom*(cpdc12*vit1 + cpdc22*vit2 + cpdc23*vit3)
-  trav(3,iel) = trav(3,iel) + romvom*(cpdc13*vit1 + cpdc23*vit2 + cpdc33*vit3)
+  trav(1,iel) = romvom*(cpdc11*vit1 + cpdc12*vit2 + cpdc13*vit3)
+  trav(2,iel) = romvom*(cpdc12*vit1 + cpdc22*vit2 + cpdc23*vit3)
+  trav(3,iel) = romvom*(cpdc13*vit1 + cpdc23*vit2 + cpdc33*vit3)
 
 enddo
 
