@@ -32,6 +32,7 @@
 !------------------------------------------------------------------------------
 !> \param[in]     nvar          total number of variables
 !> \param[in]     nscal         total number of scalars
+!> \param[in]     iterns        iteration number on Navier-Stokes equations
 !> \param[in,out] itypfb        boundary face types
 !> \param[out]    itrifb        tab d'indirection pour tri des faces
 !> \param[in,out] icodcl        face boundary condition code:
@@ -67,7 +68,7 @@
 !______________________________________________________________________________
 
 subroutine typecl &
- ( nvar   , nscal  ,                                              &
+ ( nvar   , nscal  , iterns ,                                     &
    itypfb , itrifb , icodcl , isostd ,                            &
    rcodcl )
 
@@ -99,7 +100,7 @@ implicit none
 
 ! Arguments
 
-integer          nvar   , nscal
+integer          nvar   , nscal, iterns
 
 integer          icodcl(ndimfb,nvar)
 integer          itypfb(ndimfb) , itrifb(ndimfb)
@@ -113,13 +114,12 @@ integer          ifac, ivar, iel
 integer          iok, inc, iccocg, iprev, ideb, ifin, inb, isum
 integer          ityp, ii, jj, iflmab
 integer          ifadir
-integer          iut  , ivt   , iwt, iscal
+integer          iut  , ivt   , iwt, ialt, iscal
 integer          keyvar, keycpl
 integer          iivar, icpl
-integer          f_id, i_dim, f_type, nfld, f_dim
+integer          f_id, i_dim, f_type, nfld, f_dim, f_id_yplus
 
 double precision pref
-double precision diipbx, diipby, diipbz
 double precision flumbf, flumty(ntypmx)
 double precision d0, d0min
 double precision xyzref(3)
@@ -153,18 +153,17 @@ type(var_cal_opt) :: vcopt
 allocate(pripb(ndimfb))
 
 ! Initialize variables to avoid compiler warnings
-
+pripb = 0.d0
 pref = 0.d0
 
-call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
-call field_get_val_s(iflmab, bmasfl)
-
 call field_get_val_prev_s(ivarfl(ipr), cvara_pr)
 
 call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt)
 
 call field_get_key_id("variable_id", keyvar)
 
+call field_get_id_try("wall_yplus", f_id_yplus)
+
 ! Number of fields
 call field_get_n_fields(nfld)
 
@@ -806,6 +805,8 @@ if (itbslb.gt.0) then
   inc = 1
   iccocg = 1
 
+  call grdpor(inc)
+
   call field_gradient_potential(ivarfl(ipr), iprev, imrgra, inc,      &
                                 iccocg, iphydr,                       &
                                 frcxt, grad)
@@ -1014,9 +1015,9 @@ do ivar = 1, nvar
       ifac = itrifb(ii)
       if (icodcl(ifac,ivar).eq.0) then
         icodcl(ifac,ivar)   = 5
-        ! rcodcl(ifac,ivar,1) = Utilisateur
+        ! rcodcl(ifac,ivar,1) = User
         rcodcl(ifac,ivar,2) = rinfin
-        rcodcl(ifac,ivar,3) = 0.d0
+        ! rcodcl(ifac,ivar,3) unused value, let it as default
       endif
     enddo
   elseif ((itytur.eq.2                                             &
@@ -1059,7 +1060,7 @@ do ivar = 1, nvar
   endif
 enddo
 
-! Turbulent fluxes: 0 Dirichlet if Dirichlet on the scalar
+! Turbulent fluxes
 do iscal = 1, nscal
   if (ityturt(iscal).eq.3) then
     ! Name of the scalar ivar
@@ -1068,7 +1069,6 @@ do iscal = 1, nscal
     ! Index of the corresponding turbulent flux
     call field_get_id(trim(fname)//'_turbulent_flux', f_id)
 
-
     ! Set pointer values of turbulent fluxes in icodcl
     call field_get_key_int(f_id, keyvar, iut)
     ivt = iut + 1
@@ -1076,14 +1076,7 @@ do iscal = 1, nscal
 
     do ii = ideb, ifin
       ifac = itrifb(ii)
-      if (icodcl(ifac, isca(iscal)).eq.1.and.icodcl(ifac, iut).eq.0) then
-        icodcl(ifac,iut) = 1
-        icodcl(ifac,ivt) = 1
-        icodcl(ifac,iwt) = 1
-        rcodcl(ifac,iut,1) = 0.d0
-        rcodcl(ifac,ivt,1) = 0.d0
-        rcodcl(ifac,iwt,1) = 0.d0
-      else if (icodcl(ifac, isca(iscal)).eq.5.and.icodcl(ifac, iut).eq.0) then
+      if (icodcl(ifac, iut).eq.0) then
         icodcl(ifac,iut) = 5
         icodcl(ifac,ivt) = 5
         icodcl(ifac,iwt) = 5
@@ -1093,6 +1086,28 @@ do iscal = 1, nscal
       endif
     enddo
   endif
+
+  ! EB-GGDH/AFM/DFM alpha boundary conditions
+  if (iturt(iscal).eq.11 .or. iturt(iscal).eq.21 .or. iturt(iscal).eq.31) then
+    ! Name of the scalar ivar
+    call field_get_name(ivarfl(isca(iscal)), fname)
+
+    ! Index of the corresponding turbulent flux
+    call field_get_id(trim(fname)//'_alpha', f_id)
+
+    ! Set pointer values of turbulent fluxes in icodcl
+    call field_get_key_int(f_id, keyvar, ialt)
+
+    do ii = ideb, ifin
+      ifac = itrifb(ii)
+      if (icodcl(ifac, ialt).eq.0) then
+        icodcl(ifac,ialt) = 5
+        rcodcl(ifac,ialt,1) = 0.d0
+      endif
+    enddo
+  endif
+
+! End loop over scalars
 enddo
 
 
@@ -1155,7 +1170,7 @@ do ivar = 1, nvar
   endif
 enddo
 
-! Turbulent fluxes: 0 Dirichlet if Dirichlet on the scalar
+! Turbulent fluxes
 do iscal = 1, nscal
   if (ityturt(iscal).eq.3) then
     ! Name of the scalar ivar
@@ -1164,7 +1179,6 @@ do iscal = 1, nscal
     ! Index of the corresponding turbulent flux
     call field_get_id(trim(fname)//'_turbulent_flux', f_id)
 
-
     ! Set pointer values of turbulent fluxes in icodcl
     call field_get_key_int(f_id, keyvar, iut)
     ivt = iut + 1
@@ -1172,14 +1186,7 @@ do iscal = 1, nscal
 
     do ii = ideb, ifin
       ifac = itrifb(ii)
-      if (icodcl(ifac, isca(iscal)).eq.1.and.icodcl(ifac, iut).eq.0) then
-        icodcl(ifac,iut) = 1
-        icodcl(ifac,ivt) = 1
-        icodcl(ifac,iwt) = 1
-        rcodcl(ifac,iut,1) = 0.d0
-        rcodcl(ifac,ivt,1) = 0.d0
-        rcodcl(ifac,iwt,1) = 0.d0
-      else if (icodcl(ifac, isca(iscal)).eq.6.and.icodcl(ifac, iut).eq.0) then
+      if (icodcl(ifac, iut).eq.0) then
         icodcl(ifac,iut) = 6
         icodcl(ifac,ivt) = 6
         icodcl(ifac,iwt) = 6
@@ -1189,6 +1196,28 @@ do iscal = 1, nscal
       endif
     enddo
   endif
+
+  ! EB-GGDH/AFM/DFM alpha boundary conditions
+  if (iturt(iscal).eq.11 .or. iturt(iscal).eq.21 .or. iturt(iscal).eq.31) then
+    ! Name of the scalar ivar
+    call field_get_name(ivarfl(isca(iscal)), fname)
+
+    ! Index of the corresponding turbulent flux
+    call field_get_id(trim(fname)//'_alpha', f_id)
+
+    ! Set pointer values of turbulent fluxes in icodcl
+    call field_get_key_int(f_id, keyvar, ialt)
+
+    do ii = ideb, ifin
+      ifac = itrifb(ii)
+      if (icodcl(ifac, ialt).eq.0) then
+        icodcl(ifac,ialt) = 6
+        rcodcl(ifac,ialt,1) = 0.d0
+      endif
+    enddo
+  endif
+
+! End loop over scalars
 enddo
 
 !===============================================================================
@@ -1213,6 +1242,11 @@ ifin = ifinty(ientre)
 iok = 0
 iivar = 0
 do ivar = 1, nvar
+
+  ! Convective mass flux of the variable
+  call field_get_key_int(ivarfl(ivar), kbmasf, iflmab)
+  call field_get_val_s(iflmab, bmasfl)
+
   do ii = ideb, ifin
     ifac = itrifb(ii)
     if(icodcl(ifac,ivar).eq.0) then
@@ -1241,8 +1275,8 @@ do ivar = 1, nvar
       else if (rcodcl(ifac,ivar,1).gt.rinfin*0.5d0) then
 
         flumbf = bmasfl(ifac)
-        ! Outgoing flux
-        if (flumbf.ge.-epzero) then
+        ! Outgoing flux or yplus
+        if (flumbf.ge.-epzero.or. ivarfl(ivar).eq.f_id_yplus) then
           icodcl(ifac,ivar)   = 3
           rcodcl(ifac,ivar,1) = 0.d0
           rcodcl(ifac,ivar,2) = rinfin
@@ -1292,6 +1326,11 @@ ifin = ifinty(i_convective_inlet)
 iok = 0
 iivar = 0
 do ivar = 1, nvar
+
+  ! Convective mass flux of the variable
+  call field_get_key_int(ivarfl(ivar), kbmasf, iflmab)
+  call field_get_val_s(iflmab, bmasfl)
+
   do ii = ideb, ifin
     ifac = itrifb(ii)
     if(icodcl(ifac,ivar).eq.0) then
@@ -1319,8 +1358,8 @@ do ivar = 1, nvar
       else if (rcodcl(ifac,ivar,1).gt.rinfin*0.5d0) then
 
         flumbf = bmasfl(ifac)
-        ! Outgoing flux
-        if (flumbf.ge.-epzero) then
+        ! Outgoing flux or yplus
+        if (flumbf.ge.-epzero.or. ivarfl(ivar).eq.f_id_yplus) then
           icodcl(ifac,ivar)   = 3
           rcodcl(ifac,ivar,1) = 0.d0
           rcodcl(ifac,ivar,2) = rinfin
@@ -1572,15 +1611,17 @@ enddo
 
 call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt)
 
-if(vcopt%iwarni.ge.1 .or. mod(ntcabs,ntlist).eq.0                        &
-       .or.(ntcabs.le.ntpabs+2).or.(ntcabs.ge.ntmabs-1)) then
-  write(nfecra,7010)
-endif
+! Writings: mass flux if verbosity or when log is on, and at the first two
+! iterations and the two last iteration. Only the first iteration on navstv.
+if (vcopt%iwarni.ge.1 .or. (iterns.eq.1.and.(mod(ntcabs,ntlist).eq.0 &
+  .or.(ntcabs.le.ntpabs+2).or.(ntcabs.ge.ntmabs-1)))) then
 
-!     On ecrit le flux de masse si IWARNI>0, a la periodicite NTLIST
-!     et au deux premiers et deux derniers pas de temps.
-if(vcopt%iwarni.ge.1 .or. mod(ntcabs,ntlist).eq.0                      &
-     .or.(ntcabs.le.ntpabs+2).or.(ntcabs.ge.ntmabs-1)) then
+  ! Header
+  write(nfecra,7010)
+
+  ! Convective mass flux of the velocity
+  call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
+  call field_get_val_s(iflmab, bmasfl)
 
   do ii = 1, ntypmx
     flumty(ii) = 0.d0
