@@ -42,9 +42,11 @@
 
 #include "cs_base.h"
 #include "cs_boundary_zone.h"
+#include "cs_control.h"
 #include "cs_defs.h"
 #include "cs_domain.h"
-#include "cs_domain_post.h"
+#include "cs_domain_boundary.h"
+#include "cs_domain_op.h"
 #include "cs_domain_setup.h"
 #include "cs_equation.h"
 #include "cs_gwf.h"
@@ -55,8 +57,6 @@
 #include "cs_post.h"
 #include "cs_prototypes.h"
 #include "cs_navsto_system.h"
-#include "cs_restart.h"
-#include "cs_restart_default.h"
 #include "cs_timer.h"
 #include "cs_timer_stats.h"
 #include "cs_volume_zone.h"
@@ -82,277 +82,6 @@ static int  cs_cdo_ts_id;
  * Private function prototypes
  *============================================================================*/
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Read a restart file for the CDO module
- *
- * \param[in]  domain     pointer to a cs_domain_t structure
- */
-/*----------------------------------------------------------------------------*/
-
-static void
-_read_restart(const cs_domain_t  *domain)
-{
-  CS_UNUSED(domain);
-
-  cs_restart_t  *restart = cs_restart_create("main", // restart file name
-                                             NULL,   // directory name
-                                             CS_RESTART_MODE_READ);
-
-  const char err_i_val[] = N_("Restart mismatch for: %s\n"
-                              "read: %d\n"
-                              "expected: %d.");
-  int i_val;
-
-  /* Read a new section: version */
-  int  version = 400000;
-  cs_restart_read_section(restart,
-                          "code_saturne:checkpoint:main:version", // secname
-                          CS_MESH_LOCATION_NONE,                  // ml_id
-                          1,                                      // nb. values
-                          CS_TYPE_cs_int_t,                       // val. type
-                          &i_val);                                // value(s)
-
-  if (i_val != version)
-    bft_error(__FILE__, __LINE__, 0, _(err_i_val),
-              "code_saturne:checkpoint:main:version", version, i_val);
-
-  /* Read a new section: field information */
-  cs_map_name_to_id_t  *old_field_map = NULL;
-
-  cs_restart_read_field_info(restart, &old_field_map);
-
-  /* Read a new section */
-  int  n_equations = cs_equation_get_n_equations();
-  cs_restart_read_section(restart,
-                          "cdo:n_equations",
-                          CS_MESH_LOCATION_NONE,
-                          1,
-                          CS_TYPE_cs_int_t,
-                          &i_val);
-
-  if (i_val != n_equations)
-    bft_error(__FILE__, __LINE__, 0, _(err_i_val),
-              "cdo:n_equations", n_equations, i_val);
-
-  /* Read a new section */
-  int  n_properties = cs_property_get_n_properties();
-  cs_restart_read_section(restart,
-                          "cdo:n_properties",
-                          CS_MESH_LOCATION_NONE,
-                          1,
-                          CS_TYPE_cs_int_t,
-                          &i_val);
-
-  if (i_val != n_properties)
-    bft_error(__FILE__, __LINE__, 0, _(err_i_val),
-              "cdo:n_properties", n_properties, i_val);
-
-  /* Read a new section */
-  int  n_adv_fields = cs_advection_field_get_n_fields();
-  cs_restart_read_section(restart,
-                          "cdo:n_adv_fields",
-                          CS_MESH_LOCATION_NONE,
-                          1,
-                          CS_TYPE_cs_int_t,
-                          &i_val);
-
-  if (i_val != n_adv_fields)
-    bft_error(__FILE__, __LINE__, 0, _(err_i_val),
-              "cdo:n_adv_fields", n_adv_fields, i_val);
-
-  /* Read a new section: activation or not of the groundwater flow module */
-  int  igwf = 0; // not activated by default
-  if (cs_gwf_is_activated()) igwf = 1;
-  cs_restart_read_section(restart,
-                          "groundwater_flow_module",
-                          CS_MESH_LOCATION_NONE,
-                          1,
-                          CS_TYPE_cs_int_t,
-                          &i_val);
-
-  if (i_val != igwf)
-    bft_error(__FILE__, __LINE__, 0, _(err_i_val),
-              "groundwater_flow_module", igwf, i_val);
-
-  /* Read a new section: activation or not of the Navier-Stokes system */
-  int  inss = 0; // not activated by default
-  if (cs_navsto_system_is_activated()) inss = 1;
-  cs_restart_read_section(restart,
-                          "navier_stokes_system",
-                          CS_MESH_LOCATION_NONE,
-                          1,
-                          CS_TYPE_cs_int_t,
-                          &i_val);
-
-  if (i_val != inss)
-    bft_error(__FILE__, __LINE__, 0, _(err_i_val),
-              "navier_stokes_system", inss, i_val);
-
-  /* Read a new section: computation or not of the wall distance */
-  int  iwall = 0;
-  if (cs_walldistance_is_activated()) iwall = 1;
-  cs_restart_read_section(restart,
-                          "wall_distance",
-                          CS_MESH_LOCATION_NONE,
-                          1,
-                          CS_TYPE_cs_int_t,
-                          &i_val);
-
-  if (i_val != iwall)
-    bft_error(__FILE__, __LINE__, 0, _(err_i_val),
-              "wall_distance", iwall, i_val);
-
-  /* Read a new section: number of computed time steps */
-  int  nt_cur = 0;
-  cs_restart_read_section(restart,
-                          "cur_time_step",
-                          CS_MESH_LOCATION_NONE,
-                          1,
-                          CS_TYPE_cs_int_t,
-                          &nt_cur);
-
-  /* Read a new section: number of computed time steps */
-  cs_real_t  t_cur = 0;
-  cs_restart_read_section(restart,
-                          "cur_time",
-                          CS_MESH_LOCATION_NONE,
-                          1,
-                          CS_TYPE_cs_real_t,
-                          &t_cur);
-
-  cs_time_step_redefine_cur(nt_cur, t_cur);
-
-  /* Main variables */
-  int  t_id_flag = 0; // Only current values
-  cs_restart_read_variables(restart, old_field_map, t_id_flag, NULL);
-
-  cs_map_name_to_id_destroy(&old_field_map);
-
-  // TODO: read field values for previous time step if needed
-
-  int n_fields = cs_field_n_fields();
-  for (int f_id = 0; f_id < n_fields; f_id++) {
-    cs_field_t *f = cs_field_by_id(f_id);
-    cs_field_current_to_previous(f);
-  }
-
-  /* Finalize restart process */
-  cs_restart_destroy(&restart);
-}
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Write a restart file for the CDO module
- *
- * \param[in]  domain     pointer to a cs_domain_t structure
- */
-/*----------------------------------------------------------------------------*/
-
-static void
-_write_restart(const cs_domain_t  *domain)
-{
-  cs_restart_t  *restart = cs_restart_create("main", // restart file name
-                                             NULL,   // directory name
-                                             CS_RESTART_MODE_WRITE);
-
-  /* Write a new section: version */
-  int  version = 400000;
-  cs_restart_write_section(restart,
-                           "code_saturne:checkpoint:main:version", // secname
-                           CS_MESH_LOCATION_NONE,                  // ml_id
-                           1,                                      // nb. values
-                           CS_TYPE_cs_int_t,                       // val. type
-                           &version);                              // value(s)
-
-  /* Write a new section: field information */
-  cs_restart_write_field_info(restart);
-
-  /* Write a new section */
-  int  n_equations = cs_equation_get_n_equations();
-  cs_restart_write_section(restart,
-                           "cdo:n_equations",
-                           CS_MESH_LOCATION_NONE,
-                           1,
-                           CS_TYPE_cs_int_t,
-                           &n_equations);
-
-  /* Write a new section */
-  int  n_properties = cs_property_get_n_properties();
-  cs_restart_write_section(restart,
-                           "cdo:n_properties",
-                           CS_MESH_LOCATION_NONE,
-                           1,
-                           CS_TYPE_cs_int_t,
-                           &n_properties);
-
-  /* Write a new section */
-  int  n_adv_fields = cs_advection_field_get_n_fields();
-  cs_restart_write_section(restart,
-                           "cdo:n_adv_fields",
-                           CS_MESH_LOCATION_NONE,
-                           1,
-                           CS_TYPE_cs_int_t,
-                           &n_adv_fields);
-
-  /* Write a new section: activation or not of the groundwater flow module */
-  int  igwf = 0; // not activated by default
-  if (cs_gwf_is_activated()) igwf = 1;
-  cs_restart_write_section(restart,
-                           "groundwater_flow_module",
-                           CS_MESH_LOCATION_NONE,
-                           1,
-                           CS_TYPE_cs_int_t,
-                           &igwf);
-
-  /* Write a new section: activation or not of the Navier-Stokes system */
-  int  inss = 0; // not activated by default
-  if (cs_navsto_system_is_activated()) inss = 1;
-  cs_restart_write_section(restart,
-                           "navier_stokes_system",
-                           CS_MESH_LOCATION_NONE,
-                           1,
-                           CS_TYPE_cs_int_t,
-                           &inss);
-
-  /* Write a new section: computation or not of the wall distance */
-  int  iwall = 0;
-  if (cs_walldistance_is_activated()) iwall = 1;
-  cs_restart_write_section(restart,
-                           "wall_distance",
-                           CS_MESH_LOCATION_NONE,
-                           1,
-                           CS_TYPE_cs_int_t,
-                           &iwall);
-
-  /* Write a new section: number of computed time steps */
-  int  ntcabs = domain->time_step->nt_cur;
-  cs_restart_write_section(restart,
-                           "cur_time_step",
-                           CS_MESH_LOCATION_NONE,
-                           1,
-                           CS_TYPE_cs_int_t,
-                           &ntcabs);
-
-  /* Read a new section: number of computed time steps */
-  cs_real_t  ttcabs = domain->time_step->t_cur;
-  cs_restart_write_section(restart,
-                           "cur_time",
-                           CS_MESH_LOCATION_NONE,
-                           1,
-                           CS_TYPE_cs_real_t,
-                           &ttcabs);
-
-  /* Main variables */
-  int  t_id_flag = 0; // Only current values
-  cs_restart_write_variables(restart, t_id_flag, NULL);
-
-  // TODO: write field values for previous time step if needed
-
-  /* Finalize restart process */
-  cs_restart_destroy(&restart);
-}
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Compute equations which user-defined and steady-state
@@ -376,20 +105,29 @@ _compute_steady_user_equations(cs_domain_t   *domain)
 
       if (type == CS_EQUATION_TYPE_USER) {
 
-        /* Define the algebraic system */
-        cs_equation_build_system(domain->mesh,
-                                 domain->time_step,
-                                 domain->dt_cur,
-                                 eq);
+        if (cs_equation_uses_new_mechanism(eq))
+          cs_equation_solve_steady_state(domain->mesh, eq);
 
-        /* Solve the algebraic system */
-        cs_equation_solve(eq);
+        else { /* Deprecated */
+
+          double  dt_cur = 0.; /* Useless for steady-state equations */
+
+          /* Define the algebraic system */
+          cs_equation_build_system(domain->mesh,
+                                   domain->time_step,
+                                   dt_cur,
+                                   eq);
+
+          /* Solve the algebraic system */
+          cs_equation_solve_deprecated(eq);
+
+        }
 
       } /* User-defined equation */
 
     } /* Steady-state equation */
 
-  } // Loop on equations
+  } /* Loop on equations */
 
 }
 
@@ -420,15 +158,21 @@ _compute_unsteady_user_equations(cs_domain_t   *domain,
 
         if (type == CS_EQUATION_TYPE_USER) {
 
-          /* Define the algebraic system */
-          if (cs_equation_needs_build(eq))
+          if (cs_equation_uses_new_mechanism(eq))
+            cs_equation_solve(domain->mesh, domain->dt_cur, eq);
+
+          else { /* Deprecated */
+
+            /* Define the algebraic system */
             cs_equation_build_system(domain->mesh,
                                      domain->time_step,
                                      domain->dt_cur,
                                      eq);
 
-          /* Solve domain */
-          cs_equation_solve(eq);
+            /* Solve domain */
+            cs_equation_solve_deprecated(eq);
+
+          }
 
         } /* User-defined equation */
 
@@ -440,6 +184,58 @@ _compute_unsteady_user_equations(cs_domain_t   *domain,
 
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Solve only steady-state equations
+ *
+ * \param[in, out]  domain     pointer to a cs_domain_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+static void
+_solve_steady_state_domain(cs_domain_t  *domain)
+{
+  bool  do_output = cs_domain_needs_log(domain);
+
+  /* Output information */
+  if (domain->only_steady) {
+    cs_log_printf(CS_LOG_DEFAULT, "\n%s", lsepline);
+    cs_log_printf(CS_LOG_DEFAULT, "#      Solve steady-state problem(s)\n");
+    cs_log_printf(CS_LOG_DEFAULT, "%s", lsepline);
+  }
+  else if (do_output) {
+    cs_log_printf(CS_LOG_DEFAULT, "\n%s", lsepline);
+    cs_log_printf(CS_LOG_DEFAULT,
+                  "-ite- 0; >> Solve only steady-state equations if needed");
+    cs_log_printf(CS_LOG_DEFAULT, "\n%s\n", lsepline);
+  }
+
+  /* Predefined equation for the computation of the wall distance */
+  if (cs_walldistance_is_activated())
+    cs_walldistance_compute(domain->mesh,
+                            domain->time_step,
+                            domain->connect,
+                            domain->cdo_quantities);
+
+  /* If the problem is globally unsteady, only steady-state equations are
+     solved */
+  if (cs_gwf_is_activated())
+    cs_gwf_compute_steady_state(domain->mesh,
+                                domain->time_step,
+                                domain->connect,
+                                domain->cdo_quantities);
+
+  if (cs_navsto_system_is_activated())
+    cs_navsto_system_compute_steady_state(domain->mesh);
+
+  /* User-defined equations */
+  _compute_steady_user_equations(domain);
+
+  /* Extra operations and post-processing of the computed solutions */
+  cs_domain_post(domain);
+
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Solve all the equations of a computational domain for one time step
@@ -454,75 +250,31 @@ _solve_domain(cs_domain_t  *domain)
   int  nt_cur = domain->time_step->nt_cur;
   bool  do_output = cs_domain_needs_log(domain);
 
-  /* Setup step for all equations */
-  if (nt_cur == 0) {
+  /* Output information */
+  if (do_output) {
 
-    /* Output information */
-    if (domain->only_steady) {
-      cs_log_printf(CS_LOG_DEFAULT, "\n%s", lsepline);
-      cs_log_printf(CS_LOG_DEFAULT, "#      Solve steady-state problem(s)\n");
-      cs_log_printf(CS_LOG_DEFAULT, "%s", lsepline);
-    }
-    else if (do_output) {
-      cs_log_printf(CS_LOG_DEFAULT, "\n%s", lsepline);
-      cs_log_printf(CS_LOG_DEFAULT,
-                    "-ite- %5d; time= %5.3e s; dt= %5.3e >> Solve domain\n",
-                    nt_cur, domain->time_step->t_cur, domain->dt_cur);
-      cs_log_printf(CS_LOG_DEFAULT, "%s", lsepline);
-    }
-
-    /* Predefined equation for the computation of the wall distance */
-    if (cs_walldistance_is_activated())
-      cs_walldistance_compute(domain->mesh,
-                              domain->time_step,
-                              domain->dt_cur,
-                              domain->connect,
-                              domain->cdo_quantities);
-
-    /* If unsteady, only initialization is done, otherwise one makes the whole
-       computation */
-    if (cs_gwf_is_activated())
-      cs_gwf_compute(domain->mesh,
-                     domain->time_step,
-                     domain->dt_cur,
-                     domain->connect,
-                     domain->cdo_quantities);
-
-    if (cs_navsto_system_is_activated())
-      cs_navsto_system_compute(domain->mesh, domain->dt_cur);
-
-    /* User-defined equations */
-    _compute_steady_user_equations(domain);
-
-    /* Only initialization is done */
-    _compute_unsteady_user_equations(domain, nt_cur);
+    double  t_cur = domain->time_step->t_cur;
 
+    cs_log_printf(CS_LOG_DEFAULT, "\n%s", lsepline);
+    cs_log_printf(CS_LOG_DEFAULT,
+                  "-ite- %d >> Solve domain from time=%6.4e to %6.4e;"
+                  " dt=%5.3e",
+                  nt_cur, t_cur, t_cur + domain->dt_cur, domain->dt_cur);
+    cs_log_printf(CS_LOG_DEFAULT, "\n%s", lsepline);
   }
-  else { /* nt_cur > 0: solve unsteady problems */
 
-    /* Output information */
-    if (do_output) {
-      cs_log_printf(CS_LOG_DEFAULT, "\n%s", lsepline);
-      cs_log_printf(CS_LOG_DEFAULT,
-                    "-ite- %5d; time = %5.3e s >> Solve domain\n",
-                    nt_cur, domain->time_step->t_cur);
-      cs_log_printf(CS_LOG_DEFAULT, "%s", lsepline);
-    }
+  if (cs_gwf_is_activated())
+    cs_gwf_compute(domain->mesh,
+                   domain->time_step,
+                   domain->dt_cur,
+                   domain->connect,
+                   domain->cdo_quantities);
 
-    if (cs_gwf_is_activated())
-      cs_gwf_compute(domain->mesh,
-                     domain->time_step,
-                     domain->dt_cur,
-                     domain->connect,
-                     domain->cdo_quantities);
+  if (cs_navsto_system_is_activated())
+    cs_navsto_system_compute(domain->mesh, domain->dt_cur);
 
-    if (cs_navsto_system_is_activated())
-      cs_navsto_system_compute(domain->mesh, domain->dt_cur);
-
-    /* User-defined equations */
-    _compute_unsteady_user_equations(domain, nt_cur);
-
-  }
+  /* User-defined equations */
+  _compute_unsteady_user_equations(domain, nt_cur);
 
 }
 
@@ -551,23 +303,7 @@ _log_setup(const cs_domain_t   *domain)
   cs_log_printf(CS_LOG_SETUP, "%s", lsepline);
 
   /* Boundaries of the domain */
-  cs_domain_boundary_t  *bdy = domain->boundary;
-  cs_log_printf(CS_LOG_SETUP, "\n  Domain boundary by default: %s\n",
-                cs_domain_get_boundary_name(bdy->default_type));
-
-  for (int i = 0; i < bdy->n_zones; i++) {
-
-    const cs_zone_t  *z = cs_boundary_zone_by_id(bdy->zone_ids[i]);
-
-    cs_gnum_t  n_g_elts = (cs_gnum_t)z->n_elts;
-    if (cs_glob_n_ranks > 1)
-      cs_parall_counter(&n_g_elts, 1);
-
-    cs_log_printf(CS_LOG_SETUP, " %s: %s: %u boundary faces,",
-                  z->name, cs_domain_get_boundary_name(bdy->zone_type[i]),
-                  (unsigned int)n_g_elts);
-
-  }
+  cs_domain_boundary_log_setup();
 
   /* Time step summary */
   cs_log_printf(CS_LOG_SETUP, "\n  Time step information\n");
@@ -651,20 +387,13 @@ cs_cdo_initialize_setup(cs_domain_t   *domain)
 
   cs_timer_t t0 = cs_timer_time();
 
-  /* According to the settings, add or not predefined equations:
-      >> Wall distance
-      >> Groundwater flows
-      >> Mesh deformation
-      >> Navier-Stokes system
-  */
-  cs_domain_setup_predefined_equations(domain);
-
-  /* Add variables related to user-defined and predefined equations */
-  cs_equation_create_fields();
-  cs_advection_field_create_fields();
-
-  /* Set the scheme flag for the computational domain */
-  cs_domain_set_scheme_flags(domain);
+  /* First setup stage of the cs_domain_t structure
+   * - Define extra domain boundaries
+   * - Setup predefined equations
+   * - Create fields
+   * - Define cs_sles_t structures for variable fields
+   */
+  cs_domain_initialize_setup(domain);
 
   /* Monitoring */
   cs_timer_stats_stop(cs_cdo_ts_id);
@@ -749,8 +478,8 @@ cs_cdo_finalize(cs_domain_t    *domain)
   /* Finalize user-defined extra operations */
   cs_user_cdo_end_extra_op(domain);
 
-  /* Write a restart file */
-  _write_restart(domain);
+  /* Write a restart file if needed */
+  cs_domain_write_restart(domain);
 
   /* Print monitoring information */
   cs_equation_log_monitoring();
@@ -773,6 +502,9 @@ cs_cdo_finalize(cs_domain_t    *domain)
   /* Free common structures relatated to equations */
   cs_equation_common_free(domain->cdo_context);
 
+  /* Set flag to OFF */
+  cs_domain_set_cdo_mode(domain, CS_DOMAIN_CDO_MODE_OFF);
+
   cs_log_printf(CS_LOG_DEFAULT,
                 "\n  Finalize and free CDO-related structures.\n");
 
@@ -790,26 +522,38 @@ cs_cdo_finalize(cs_domain_t    *domain)
 void
 cs_cdo_main(cs_domain_t   *domain)
 {
-  assert(cs_domain_get_cdo_mode(domain) != CS_DOMAIN_CDO_MODE_OFF);
+  if (cs_domain_get_cdo_mode(domain) == CS_DOMAIN_CDO_MODE_OFF)
+    return;
 
   cs_timer_t t0 = cs_timer_time();
 
   /* Timer statistics */
   cs_timer_stats_start(cs_cdo_ts_id);
 
-  if (cs_restart_present())
-    _read_restart(domain);
-
   /*  Build high-level structures and create algebraic systems
       Set the initial values of the fields and properties */
   cs_domain_initialize_systems(domain);
 
+  /* Read a restart file if needed */
+  cs_domain_read_restart(domain);
+
+  /* Activate writers for post-processing */
+  cs_post_activate_writer(CS_POST_WRITER_ALL_ASSOCIATED, true);
+
   /* Initialization for user-defined extra operations. Should be done
-     after the domain initializatoin if one wants to overwrite the field
+     after the domain initialization if one wants to overwrite the field
      initialization for instance */
   cs_user_cdo_start_extra_op(cs_glob_domain);
 
-  while (cs_domain_needs_iteration(domain)) { // Main time loop
+  /* Build and solve equations related to the computational domain in case of
+     steady-state equations */
+  _solve_steady_state_domain(domain);
+
+  /* Main time loop */
+  if (domain->time_step->nt_cur == 0)
+    domain->time_step->nt_cur = 1; /* Same numbering as Finite Volume part */
+
+  while (cs_domain_needs_iteration(domain)) {
 
     /* Define the current time step */
     cs_domain_define_current_time_step(domain);
@@ -817,11 +561,21 @@ cs_cdo_main(cs_domain_t   *domain)
     /* Build and solve equations related to the computational domain */
     _solve_domain(domain);
 
+    /* Increment time */
+    cs_domain_increment_time(domain);
+
     /* Extra operations and post-processing of the computed solutions */
     cs_domain_post(domain);
 
     /* Increment time */
-    cs_domain_increment_time(domain);
+    cs_domain_increment_time_step(domain);
+
+    /* Read a control file if present */
+    cs_control_check_file();
+
+    /* Add a checkpoint if needed */
+    cs_domain_write_restart(domain);
+
     cs_timer_stats_increment_time_step();
 
   }
