@@ -123,8 +123,7 @@ cs_cdo_time_get_scheme_function(const cs_flag_t             sys_flag,
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Update the RHS with the previously computed array of values (for
- *         instance the previous source term evaluation)
+ * \brief  Update the RHS with the previously computed array of values
  *         Do not use OpenMP inside this function since it may be called from
  *         an OpenMP block
  *
@@ -158,7 +157,7 @@ cs_cdo_time_update_rhs(const cs_equation_param_t    *eqp,
 
         for (cs_lnum_t i = 0; i < n_dofs; i++)
           for (int k = 0; k < stride; k++)
-            rhs[stride*i+k] += values[stride*dof_ids[i]+k];
+            rhs[stride*i+k] += values[dof_ids[stride*i+k]];
 
       }
       else {
@@ -175,7 +174,7 @@ cs_cdo_time_update_rhs(const cs_equation_param_t    *eqp,
         if (stride > 1) {
           for (cs_lnum_t i = 0; i < n_dofs; i++)
             for (int k = 0; k < stride; k++)
-              rhs[stride*i+k] += tcoef * values[stride*dof_ids[i]+k];
+              rhs[stride*i+k] += tcoef * values[dof_ids[stride*i+k]];
         }
         else {
           for (cs_lnum_t i = 0; i < n_dofs; i++)
@@ -253,13 +252,7 @@ cs_cdo_time_diag_imp(const cs_equation_param_t  *eqp,
   assert(csys->n_dofs == adr->n_rows);
   assert(system_flag & CS_FLAG_SYS_TIME_DIAG);
 
-  /* STEP1 >> Apply source term contribution
-           >> RHS has already the BC contribution */
-  if (cs_equation_param_has_sourceterm(eqp))
-    for (short int i = 0; i < csys->n_dofs; i++)
-      csys->rhs[i] += csys->source[i]; // Values at t_(n+1)
-
-  /* STEP2 >> Compute the time contribution to the RHS: Mtime*pn
+  /* STEPS >> Compute the time contribution to the RHS: Mtime*pn
            >> Update the cellwise system with the time matrix
            >> Apply other contributions to the RHS */
   for (short int i = 0; i < csys->n_dofs; i++) {
@@ -291,12 +284,12 @@ cs_cdo_time_diag_imp(const cs_equation_param_t  *eqp,
 /*----------------------------------------------------------------------------*/
 
 void
-cs_cdo_time_imp(const cs_equation_param_t  *eqp,
-                const double                tpty_val,
-                const cs_sdm_t             *mass_mat,
-                const cs_flag_t             system_flag,
-                cs_cell_builder_t          *cb,
-                cs_cell_sys_t              *csys)
+cs_cdo_time_imp(const cs_equation_param_t   *eqp,
+                const double                 tpty_val,
+                const cs_sdm_t              *mass_mat,
+                const cs_flag_t              system_flag,
+                cs_cell_builder_t           *cb,
+                cs_cell_sys_t               *csys)
 {
   CS_UNUSED(system_flag);
 
@@ -308,14 +301,7 @@ cs_cdo_time_imp(const cs_equation_param_t  *eqp,
   assert(mass_mat != NULL);
   assert(mass_mat->n_rows == adr->n_rows);
 
-  /* STEP1 >> Apply source term contribution
-           >> RHS has already the BC contribution (+Source term at iter n
-              if required) */
-  if (cs_equation_param_has_sourceterm(eqp))
-    for (short int i = 0; i < csys->n_dofs; i++)
-      csys->rhs[i] += csys->source[i]; // Values at t_(n+1)
-
-  /* STEP2 >> Compute the time contribution to the RHS: Mtime*pn
+  /* STEPS >> Compute the time contribution to the RHS: Mtime*pn
            >> Update the cellwise system with the time matrix
            >> Apply other contributions to the RHS */
 
@@ -497,18 +483,13 @@ cs_cdo_time_diag_theta(const cs_equation_param_t  *eqp,
   assert(csys->n_dofs == adr->n_rows);
   assert(mass_mat != NULL);
 
-  /* STEP1 >> Treatment of the source term */
-  if (cs_equation_param_has_sourceterm(eqp))
-    for (short int i = 0; i < csys->n_dofs; i++)
-      csys->rhs[i] += eqp->theta * csys->source[i];
-
-  /* STEP2 >> Compute the contribution of the "adr" to the RHS: tcoef*adr_pn */
+  /* STEPS >> Compute the contribution of the "adr" to the RHS: tcoef*adr_pn */
   double  *adr_pn = cb->values;
   cs_sdm_square_matvec(adr, csys->val_n, adr_pn);
   for (short int i = 0; i < csys->n_dofs; i++)
     adr_pn[i] *= tcoef; // (1 - theta)
 
-  /* STEP3 >> Compute the time contribution to the RHS: Mtime*pn
+  /* STEPS >> Compute the time contribution to the RHS: Mtime*pn
            >> Update the cellwise system with the time matrix */
   double  *time_pn = cb->values + csys->n_dofs;
   for (short int i = 0; i < csys->n_dofs; i++) {
@@ -528,7 +509,7 @@ cs_cdo_time_diag_theta(const cs_equation_param_t  *eqp,
 
   }
 
-  /* STEP4 >> Apply other contributions to the RHS */
+  /* STEPS >> Apply other contributions to the RHS */
   for (short int i = 0; i < csys->n_dofs; i++)
     csys->rhs[i] += time_pn[i] - adr_pn[i];
 
@@ -568,18 +549,13 @@ cs_cdo_time_theta(const cs_equation_param_t  *eqp,
   assert(mass_mat != NULL);
   assert(mass_mat->n_rows == adr->n_rows);
 
-  /* STEP1 >> Treatment of the source term */
-  if (cs_equation_param_has_sourceterm(eqp))
-    for (short int i = 0; i < csys->n_dofs; i++)
-      csys->rhs[i] += eqp->theta * csys->source[i];
-
-  /* STEP2 >> Compute the contribution of the "adr" to the RHS: tcoef*adr_pn */
+  /* STEPS >> Compute the contribution of the "adr" to the RHS: tcoef*adr_pn */
   double  *adr_pn = cb->values;
   cs_sdm_square_matvec(adr, csys->val_n, adr_pn);
   for (short int i = 0; i < csys->n_dofs; i++)
     adr_pn[i] *= tcoef; // (1 - theta)
 
-  /* STEP3 >> Update the cellwise system with the time matrix */
+  /* STEPS >> Update the cellwise system with the time matrix */
   for (short int i = 0; i < csys->n_dofs; i++) {
 
     const int  shift_i = i*csys->n_dofs;
