@@ -40,7 +40,6 @@
 #include "bft_mem.h"
 
 #include "cs_boundary_zone.h"
-#include "cs_domain_post.h"
 #include "cs_evaluate.h"
 #include "cs_equation.h"
 #include "cs_equation_common.h"
@@ -101,201 +100,8 @@ static const char _err_empty_cdo_context[] =
  * Private function prototypes
  *============================================================================*/
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Build the list of boundary faces attached to a wall boundary
- *         condition
- *         Function pointer to mesh location elements selection definition.
- *
- * If non-empty and not containing all elements, a list of elements
- * of the parent mesh belonging to the location should be allocated
- * (using BFT_MALLOC) and defined by this function when called.
- * This list's lifecycle is then managed by the mesh location object.
- *
- * \param[in]   input        pointer to a structure cast on-the-fly
- * \param[in]   m            pointer to associated mesh structure.
- * \param[in]   location_id  id of associated location.
- * \param[out]  n_elts       number of selected elements
- * \param[out]  elt_list     list of selected elements.
- */
-/*----------------------------------------------------------------------------*/
-
-static void
-_wall_boundary_selection(void              *input,
-                         const cs_mesh_t   *m,
-                         int                location_id,
-                         cs_lnum_t         *n_elts,
-                         cs_lnum_t        **elt_ids)
-{
-  CS_UNUSED(location_id);
-
-  cs_domain_t  *domain = (cs_domain_t *)input;
-
-  /* Handle case where global domain has a temporary existence
-     and has been destroyed */
-
-  if (domain == NULL) {
-    *n_elts = 0;
-    *elt_ids = NULL;
-    return;
-  }
-
-  cs_domain_boundary_t  *db = domain->boundary;
-  cs_lnum_t  n_wall_elts = 0;
-  cs_lnum_t *wall_elts = NULL;
-  bool  *is_wall = NULL;
-
-  BFT_MALLOC(is_wall, m->n_b_faces, bool);
-
-  if (db->default_type == CS_DOMAIN_BOUNDARY_WALL) {
-
-    for (cs_lnum_t i = 0; i < m->n_b_faces; i++)
-      is_wall[i] = true;
-
-    for (int i = 0; i < db->n_zones; i++) {
-      if (db->zone_type[i] != CS_DOMAIN_BOUNDARY_WALL) {
-
-        int z_id = db->zone_ids[i];
-        const cs_zone_t  *z = cs_boundary_zone_by_id(z_id);
-        const cs_lnum_t  _n_faces = z->n_elts;
-        const cs_lnum_t  *_face_ids = z->elt_ids;
-
-        for (cs_lnum_t j = 0; j < _n_faces; j++)
-          is_wall[_face_ids[j]] = false;
-
-      }
-    }
-
-  }
-  else { /* Wall is not the default boundary */
-
-    for (cs_lnum_t i = 0; i < m->n_b_faces; i++)
-      is_wall[i] = false;
-
-    for (int i = 0; i < db->n_zones; i++) {
-      if (db->zone_type[i] == CS_DOMAIN_BOUNDARY_WALL) {
-
-        int z_id = db->zone_ids[i];
-        const cs_zone_t  *z = cs_boundary_zone_by_id(z_id);
-        const cs_lnum_t  _n_faces = z->n_elts;
-        const cs_lnum_t  *_face_ids = z->elt_ids;
-
-        for (cs_lnum_t j = 0; j < _n_faces; j++)
-          is_wall[_face_ids[j]] = true;
-
-      }
-    }
-
-  } /* Which default ? */
-
-  /* Count  */
-  for (cs_lnum_t i = 0; i < m->n_b_faces; i++)
-    if (is_wall[i])
-      n_wall_elts++;
-
-  if (n_wall_elts < m->n_b_faces) {
-
-    /* Fill list  */
-    BFT_MALLOC(wall_elts, n_wall_elts, cs_lnum_t);
-
-    cs_lnum_t shift = 0;
-    for (cs_lnum_t i = 0; i < m->n_b_faces; i++)
-      if (is_wall[i])
-        wall_elts[shift++] = i;
-
-    assert(shift == n_wall_elts);
-
-  }
-
-  BFT_FREE(is_wall);
-
-  /* Return pointers */
-  *n_elts = n_wall_elts;
-  *elt_ids = wall_elts;
-}
-
 /*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
-/*============================================================================
- * Public function prototypes
- *============================================================================*/
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Set to true the automatic update of all advection fields
- *
- * \param[in, out]  domain    pointer to a cs_domain_t structure
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_domain_update_advfield(cs_domain_t       *domain)
-{
-  if (domain == NULL)
-    bft_error(__FILE__, __LINE__, 0, _err_empty_domain);
-  if (domain->cdo_context == NULL)
-    bft_error(__FILE__, __LINE__, 0, _err_empty_cdo_context);
-
-  domain->cdo_context->force_advfield_update = true;
-}
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Add a new zone gathering all CS_DOMAIN_BOUNDARY_WALL zone type
- *
- * \param[in]   domain    pointer to a cs_domain_t structure
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_domain_update_wall_zones(cs_domain_t   *domain)
-{
-  /* Add a new boundary zone (and also a new mesh location) related to all
-     wall boundary faces */
-  const char  zone_name[] = "domain_walls";
-
-  int flag = CS_BOUNDARY_ZONE_WALL | CS_BOUNDARY_ZONE_PRIVATE;
-
-  int  z_id = cs_boundary_zone_define_by_func(zone_name,
-                                              _wall_boundary_selection,
-                                              domain,
-                                              flag);
-
-  /* Allow overlay with other boundary zones used to set BCs on transport
-     equations for instance (not really needed since zone is private) */
-  cs_boundary_zone_set_overlay(z_id, true);
-}
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief  Setup predefined equations which are activated.
- *         At this stage, no equation is added and the space discretization
- *         scheme and the related numerical parameters are set.
- *
- * \param[in, out]   domain    pointer to a cs_domain_t structure
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_domain_setup_predefined_equations(cs_domain_t   *domain)
-{
-  /* Wall distance */
-  if (cs_walldistance_is_activated())
-    cs_walldistance_setup();
-
-  /* Mesh deformation */
-  if (cs_mesh_deform_is_activated())
-    cs_mesh_deform_setup(domain);
-
-  /* Groundwater flow module */
-  if (cs_gwf_is_activated())
-    cs_gwf_init_setup();
-
-  /* Navier-Stokes system */
-  if (cs_navsto_system_is_activated())
-    cs_navsto_system_init_setup();
-}
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief  Define the scheme flags for the current computational domain
@@ -305,8 +111,8 @@ cs_domain_setup_predefined_equations(cs_domain_t   *domain)
  */
 /*----------------------------------------------------------------------------*/
 
-void
-cs_domain_set_scheme_flags(cs_domain_t    *domain)
+static void
+_set_scheme_flags(cs_domain_t    *domain)
 {
   if (domain == NULL)
     bft_error(__FILE__, __LINE__, 0, _err_empty_domain);
@@ -394,7 +200,7 @@ cs_domain_set_scheme_flags(cs_domain_t    *domain)
                   " Please check your settings."), cs_equation_get_name(eq));
     }
 
-  } // Loop on equations
+  } /* Loop on equations */
 
   /* Navier-Stokes sytem */
   if (cs_navsto_system_is_activated()) {
@@ -430,9 +236,202 @@ cs_domain_set_scheme_flags(cs_domain_t    *domain)
 
 }
 
+/*============================================================================
+ * Public function prototypes
+ *============================================================================*/
+
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief  Build a cs_domain_t structure
+ * \brief  Set to true the automatic update of all advection fields
+ *
+ * \param[in, out]  domain    pointer to a \ref cs_domain_t structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_domain_update_advfield(cs_domain_t       *domain)
+{
+  if (domain == NULL)
+    bft_error(__FILE__, __LINE__, 0, _err_empty_domain);
+  if (domain->cdo_context == NULL)
+    bft_error(__FILE__, __LINE__, 0, _err_empty_cdo_context);
+
+  domain->cdo_context->force_advfield_update = true;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Set auxiliary parameters related to the way output is done
+ *
+ * \param[in, out]  domain       pointer to a cs_domain_t structure
+ * \param[in]       nt_interval  frequency for the restart process
+ * \param[in]       nt_list      output frequency into the listing
+ * \param[in]       verbosity    level of information displayed
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_domain_set_output_param(cs_domain_t       *domain,
+                           int                nt_interval,
+                           int                nt_list,
+                           int                verbosity)
+{
+  if (domain == NULL) bft_error(__FILE__, __LINE__, 0, _err_empty_domain);
+
+  domain->restart_nt = nt_interval;
+  domain->output_nt = nt_list;
+  if (domain->output_nt == 0)
+    domain->output_nt = -1;
+
+  domain->verbosity = verbosity;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Set parameters for unsteady computations: the max number of time
+ *         steps or the final physical time of the simulation
+ *
+ * \param[in, out]  domain    pointer to a \ref cs_domain_t structure
+ * \param[in]       nt_max    max. number of time step iterations
+ * \param[in]       t_max     final physical time of the simulation
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_domain_set_time_param(cs_domain_t       *domain,
+                         int                nt_max,
+                         double             t_max)
+{
+  if (domain == NULL)  bft_error(__FILE__, __LINE__, 0, _err_empty_domain);
+
+  domain->time_step->nt_max = nt_max;
+  domain->time_step->t_max = t_max;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Define the value of the time step thanks to a predefined function
+ *
+ * \param[in, out] domain      pointer to a cs_domain_t structure
+ * \param[in]      func        pointer to a cs_timestep_func_t function
+ * \param[in]      func_input  pointer to a structure cast on-the-fly
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_domain_def_time_step_by_function(cs_domain_t          *domain,
+                                    cs_timestep_func_t   *func,
+                                    void                 *func_input)
+{
+  if (domain == NULL) bft_error(__FILE__, __LINE__, 0, _err_empty_domain);
+
+  domain->time_step->is_variable = 1; // not constant time step
+  domain->time_options.idtvar = 1;    /* uniform in space but can change
+                                         from one time step to the other */
+
+  cs_xdef_timestep_input_t  def = {.input = func_input,
+                                   .func = func};
+
+  domain->time_step_def = cs_xdef_timestep_create(CS_XDEF_BY_TIME_FUNCTION,
+                                                  0, // state flag
+                                                  0, // meta flag
+                                                  &def);
+
+  /* Default initialization.
+     To be changed at first call to cs_domain_time_step_increment() */
+
+  domain->dt_cur = domain->time_step->t_max;
+  domain->time_options.dtref =  domain->time_step->t_max;
+  domain->time_options.dtmin =  domain->time_step->t_max;
+  domain->time_options.dtmax = 0.;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Define the value of the time step.
+ *
+ * \param[in, out]   domain    pointer to a cs_domain_t structure
+ * \param[in]        dt        value of the constant time step
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_domain_def_time_step_by_value(cs_domain_t   *domain,
+                                 double         dt)
+{
+  if (domain == NULL) bft_error(__FILE__, __LINE__, 0, _err_empty_domain);
+
+  domain->time_step->is_variable = 0; // constant time step
+  domain->time_options.idtvar = 0;    // constant time step by default
+
+  domain->time_step_def = cs_xdef_timestep_create(CS_XDEF_BY_VALUE,
+                                                  0, // state flag
+                                                  0, // meta flag
+                                                  &dt);
+
+  domain->dt_cur = dt;
+  domain->time_options.dtref = domain->dt_cur;
+  domain->time_options.dtmin = domain->dt_cur;
+  domain->time_options.dtmax = domain->dt_cur;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  First setup stage of the cs_domain_t structure
+ *         Define extra domain boundaries
+ *         Setup predefined equations
+ *         Create fields
+ *         Define cs_sles_t structures for variable fields
+ *
+ * \param[in, out]  domain    pointer to a cs_domain_t struct.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_domain_initialize_setup(cs_domain_t    *domain)
+{
+  /* Add a boundary zone gathering all "wall" boundaries */
+  if (cs_navsto_system_is_activated() ||
+      cs_walldistance_is_activated())
+    cs_domain_boundary_def_wall_zones();
+
+  /* Setup predefined equations which are activated. At this stage,
+   * no equation is added. Space discretization scheme and the related
+   * numerical parameters are set.
+   */
+
+  /* Wall distance */
+  if (cs_walldistance_is_activated())
+    cs_walldistance_setup();
+
+  /* Mesh deformation */
+  if (cs_mesh_deform_is_activated())
+    cs_mesh_deform_setup(domain);
+
+  /* Groundwater flow module */
+  if (cs_gwf_is_activated())
+    cs_gwf_init_setup();
+
+  /* Navier-Stokes system */
+  if (cs_navsto_system_is_activated())
+    cs_navsto_system_init_setup();
+
+  /* Add variables related to user-defined and predefined equations */
+  cs_equation_create_fields();
+  cs_advection_field_create_fields();
+
+  /* Set the scheme flag for the computational domain */
+  _set_scheme_flags(domain);
+
+  /* Proceed to the settings of a cs_equation_t structure. Setup the structure
+     related to cs_sles_*
+  */
+  cs_equation_set_linear_solvers();
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Last setup stage of the cs_domain_t structure
  *
  * \param[in, out]  domain            pointer to a cs_domain_t struct.
  * \param[in, out]  mesh              pointer to a cs_mesh_t struct.
@@ -450,6 +449,13 @@ cs_domain_finalize_setup(cs_domain_t                 *domain,
   if (domain->cdo_context == NULL)
     bft_error(__FILE__, __LINE__, 0, _err_empty_cdo_context);
 
+  /* Manage checkpoint/restart settings
+   * Use the same default values for t_interval and wt_interval as the FV */
+  double  t_interval = -1.0, wt_interval = -1.0;
+  cs_restart_checkpoint_set_defaults(domain->restart_nt,
+                                     t_interval,
+                                     wt_interval);
+
   domain->mesh = mesh;
   domain->mesh_quantities = mesh_quantities;
 
@@ -470,20 +476,13 @@ cs_domain_finalize_setup(cs_domain_t                 *domain,
   /* Shared main generic structure
      Avoid the declaration of global variables by sharing pointers */
   cs_source_term_set_shared_pointers(domain->cdo_quantities,
-                                     domain->connect,
-                                     domain->time_step);
-
+                                     domain->connect);
   cs_evaluate_set_shared_pointers(domain->cdo_quantities,
-                                  domain->connect,
-                                  domain->time_step);
-
+                                  domain->connect);
   cs_property_set_shared_pointers(domain->cdo_quantities,
-                                  domain->connect,
-                                  domain->time_step);
-
+                                  domain->connect);
   cs_advection_field_set_shared_pointers(domain->cdo_quantities,
-                                         domain->connect,
-                                         domain->time_step);
+                                         domain->connect);
 
   /* Groundwater flow module */
   if (cs_gwf_is_activated()) {
@@ -506,21 +505,29 @@ cs_domain_finalize_setup(cs_domain_t                 *domain,
                               domain->time_step,
                               domain->cdo_context);
 
+  /* Set the range set structure for synchronization in parallel computing */
+  cs_equation_assign_range_set(domain->connect);
+
   /* Set the definition of user-defined properties and/or advection
-     fields (no more fields are created at this stage) */
+   * fields (no more fields are created at this stage)
+   * Last setting stage for equations: Associate properties to activate or not
+   * terms such as:
+   * --> Unsteady terms (link with a property)
+   * --> Diffusion terms (link with a property)
+   * --> Advection term  (link with an advection field)
+   * --> Reaction term (link with a property)
+   * --> Source term
+   */
+
   cs_user_cdo_finalize_setup(cs_glob_domain);
 
-  /* Proceed to the last settings of a cs_equation_t structure
-     - Assign to a cs_equation_t structure a list of function to manage this
-       structure during the computation.
-     - The set of functions chosen for each equation depends on the parameters
-       specifying the cs_equation_t structure
-     - Setup the structure related to cs_sles_*
-  */
-
-  domain->only_steady = cs_equation_finalize_setup(domain->connect,
-                                                   domain->profiling);
+  /* Assign to a cs_equation_t structure a list of function to manage this
+   * structure during the computation.
+   * The set of functions chosen for each equation depends on the parameters
+   * specifying the cs_equation_t structure
+   */
 
+  domain->only_steady = cs_equation_assign_functions();
   if (domain->only_steady)
     domain->is_last_iter = true;
 
@@ -537,11 +544,15 @@ cs_domain_finalize_setup(cs_domain_t                 *domain,
     cs_gwf_finalize_setup(domain->connect, domain->cdo_quantities);
 
   if (cs_navsto_system_is_activated())
-    cs_navsto_system_finalize_setup(domain->connect, domain->cdo_quantities);
+    cs_navsto_system_finalize_setup(domain->connect,
+                                    domain->cdo_quantities,
+                                    domain->time_step);
 
   /* Last stage to define properties (when complex definition is requested) */
   cs_property_finalize_setup();
 
+  /* Last stage to define properties (when complex definition is requested) */
+  cs_advection_field_finalize_setup();
 }
 
 /*----------------------------------------------------------------------------*/
@@ -567,11 +578,15 @@ cs_domain_initialize_systems(cs_domain_t   *domain)
                          domain->time_step);
 
   /* Set the initial condition for all advection fields */
-  cs_advection_field_update(false); // operate current to previous ?
+  cs_advection_field_update(domain->time_step->t_cur,
+                            false); /* operate current to previous ? */
 
   /* Set the initial state for the groundawater flow module */
   if (cs_navsto_system_is_activated())
-    cs_navsto_system_initialize();
+    cs_navsto_system_initialize(domain->mesh,
+                                domain->connect,
+                                domain->cdo_quantities,
+                                domain->time_step);
 
   /* Set the initial state for the groundawater flow module */
   if (cs_gwf_is_activated())
@@ -579,7 +594,8 @@ cs_domain_initialize_systems(cs_domain_t   *domain)
                   domain->connect,
                   domain->cdo_quantities,
                   domain->time_step,
-                  false); // operate current to previous ?
+                  domain->dt_cur,
+                  false); /* operate current to previous ? */
 }
 
 /*----------------------------------------------------------------------------*/
