@@ -161,6 +161,9 @@ typedef enum {
  * \enum cs_param_time_scheme_t
  *   Type of numerical scheme for the discretization in time
  *
+ * \var CS_TIME_SCHEME_STEADY
+ * No time scheme. Steady-state computation.
+ *
  * \var CS_TIME_SCHEME_IMPLICIT
  * fully implicit (forward Euler/theta-scheme = 1)
  *
@@ -176,6 +179,7 @@ typedef enum {
 
 typedef enum {
 
+  CS_TIME_SCHEME_STEADY,
   CS_TIME_SCHEME_IMPLICIT,
   CS_TIME_SCHEME_EXPLICIT,
   CS_TIME_SCHEME_CRANKNICO,
@@ -221,8 +225,12 @@ typedef enum {
  * Continuous Interior Penalty discretization. Only available for
  * \ref CS_SPACE_SCHEME_CDOVCB
  *
- * \var CS_PARAM_ADVECTION_SCHEME_UPWIND
- * Low order upwind discretization
+ * \var CS_PARAM_ADVECTION_SCHEME_MIX_CENTERED_UPWIND
+ * Centered discretization with a portion between [0,1] of upwinding.
+ * The portion is specified thanks to \ref CS_EQKEY_ADV_UPWIND_PORTION
+ * If the portion is equal to 0, then one recovers
+ * \ref CS_PARAM_ADVECTION_SCHEME_CENTERED. If the portion is equal to 1, then
+ * one recovers CS_PARAM_ADVECTION_SCHEME_UPWIND
  *
  * \var CS_PARAM_ADVECTION_SCHEME_SAMARSKII
  * Weighting between an upwind and a centered discretization relying on the
@@ -231,15 +239,19 @@ typedef enum {
  * \var CS_PARAM_ADVECTION_SCHEME_SG
  * Weighting between an upwind and a centered discretization relying on the
  * Peclet number. Weighting function = Scharfetter-Gummel
+ *
+ * \var CS_PARAM_ADVECTION_SCHEME_UPWIND
+ * Low order upwind discretization
  */
 
 typedef enum {
 
   CS_PARAM_ADVECTION_SCHEME_CENTERED,
   CS_PARAM_ADVECTION_SCHEME_CIP,
-  CS_PARAM_ADVECTION_SCHEME_UPWIND,
+  CS_PARAM_ADVECTION_SCHEME_MIX_CENTERED_UPWIND,
   CS_PARAM_ADVECTION_SCHEME_SAMARSKII,
   CS_PARAM_ADVECTION_SCHEME_SG,
+  CS_PARAM_ADVECTION_SCHEME_UPWIND,
 
   CS_PARAM_N_ADVECTION_SCHEMES
 
@@ -285,11 +297,12 @@ typedef enum {
  * Type of method for enforcing the boundary conditions. According to the type
  * of numerical scheme, some enforcements are not available.
  *
- * \var CS_PARAM_BC_ENFORCE_STRONG
- * Strong enforcement of the boundary conditions. Degrees of freedom related
- * to this boundary condition are removed from the algebraic system.
+ * \var CS_PARAM_BC_ENFORCE_ALGEBRAIC
+ * Weak enforcement of the boundary conditions (i.e. one keeps the degrees of
+ * freedom in the algebraic system) with an algebraic manipulation of the
+ * linear system
  *
- * \var CS_PARAM_BC_ENFORCE_WEAK_PENA
+ * \var CS_PARAM_BC_ENFORCE_PENALIZED
  * Weak enforcement of the boundary conditions (i.e. one keeps the degrees of
  * freedom in the algebraic system) with a penalization technique using a huge
  * value.
@@ -298,19 +311,21 @@ typedef enum {
  * Weak enforcement of the boundary conditions (i.e. one keeps the degrees of
  * freedom in the algebraic system) with a Nitsche-like penalization technique.
  * This technique does not increase the conditioning number as much as
- * \ref CS_PARAM_BC_ENFORCE_WEAK_PENA but is more computationally intensive.
+ * \ref CS_PARAM_BC_ENFORCE_PENALIZED but is more computationally intensive.
+ * The computation of the diffusion term should be activated with this choice.
  *
  * \var CS_PARAM_BC_ENFORCE_WEAK_SYM
  * Weak enforcement of the boundary conditions (i.e. one keeps the degrees of
  * freedom in the algebraic system) with a Nitsche-like penalization technique.
  * This variant enables to keep the symmetry of the algebraic contrary to
  * \ref CS_PARAM_BC_ENFORCE_WEAK_NITSCHE.
+ * The computation of the diffusion term should be activated with this choice.
  */
 
 typedef enum {
 
-  CS_PARAM_BC_ENFORCE_STRONG,
-  CS_PARAM_BC_ENFORCE_WEAK_PENA,
+  CS_PARAM_BC_ENFORCE_ALGEBRAIC,
+  CS_PARAM_BC_ENFORCE_PENALIZED,
   CS_PARAM_BC_ENFORCE_WEAK_NITSCHE,
   CS_PARAM_BC_ENFORCE_WEAK_SYM,
 
@@ -323,6 +338,23 @@ typedef enum {
  * @name Settings for the linear solvers
  * @{
  *
+ * \enum cs_param_amg_type_t
+ * Type of AMG (Algebraic MultiGrid) algorithm to use (either as a
+ * preconditionnerwith or a solver).
+ */
+
+typedef enum {
+
+  CS_PARAM_AMG_NONE,      /*!< No specified algorithm */
+  CS_PARAM_AMG_BOOMER,    /*!< Boomer algorithm from Hypre library */
+  CS_PARAM_AMG_GAMG  ,    /*!< GAMG algorithm from PETSc */
+  CS_PARAM_AMG_HOUSE_V,   /*!< In-house algorithm with V-cycle */
+  CS_PARAM_AMG_HOUSE_K,   /*!< In-house algorithm with K-cycle */
+  CS_PARAM_N_AMG_TYPES
+
+} cs_param_amg_type_t;
+
+/*!
  * \enum cs_param_precond_type_t
  * Type of preconditionner to use with the iterative solver. Some
  * preconditionners as \ref CS_PARAM_PRECOND_ILU0, \ref CS_PARAM_PRECOND_ICC0 or
@@ -335,6 +367,7 @@ typedef enum {
   CS_PARAM_PRECOND_DIAG,    /*!< Diagonal (or Jacobi) preconditioning */
   CS_PARAM_PRECOND_BJACOB,  /*!< Block Jacobi */
   CS_PARAM_PRECOND_POLY1,   /*!< Neumann polynomial preconditioning (Order 1) */
+  CS_PARAM_PRECOND_POLY2,   /*!< Neumann polynomial preconditioning (Order 2) */
   CS_PARAM_PRECOND_SSOR,    /*!< Symmetric Successive OverRelaxations */
   CS_PARAM_PRECOND_ILU0,    /*!< Incomplete LU factorization */
   CS_PARAM_PRECOND_ICC0,    /*!< Incomplete Cholesky factorization */
@@ -352,13 +385,16 @@ typedef enum {
 
 typedef enum {
 
-  CS_PARAM_ITSOL_JACOBI,    /*!< Jacobi */
-  CS_PARAM_ITSOL_CG,        /*!< Conjuguate Gradient */
-  CS_PARAM_ITSOL_BICG,      /*!< Bi-Conjuguate gradient */
-  CS_PARAM_ITSOL_BICGSTAB2, /*!< Stabilized Bi-Conjuguate gradient */
-  CS_PARAM_ITSOL_CR3,       /*!< 3-layer conjugate residual*/
-  CS_PARAM_ITSOL_GMRES,     /*!< Generalized Minimal RESidual */
-  CS_PARAM_ITSOL_AMG,       /*!< Algebraic MultiGrid */
+  CS_PARAM_ITSOL_JACOBI,           /*!< Jacobi */
+  CS_PARAM_ITSOL_GAUSS_SEIDEL,     /*!< Gauss-Seidel */
+  CS_PARAM_ITSOL_SYM_GAUSS_SEIDEL, /*!< Symetric Gauss-Seidel */
+  CS_PARAM_ITSOL_CG,               /*!< Conjuguate Gradient */
+  CS_PARAM_ITSOL_BICG,             /*!< Bi-Conjuguate gradient */
+  CS_PARAM_ITSOL_BICGSTAB2,        /*!< Stabilized Bi-Conjuguate gradient */
+  CS_PARAM_ITSOL_CR3,              /*!< 3-layer conjugate residual*/
+  CS_PARAM_ITSOL_GMRES,            /*!< Generalized Minimal RESidual */
+  CS_PARAM_ITSOL_FCG,              /*!< Flexible Conjuguate Gradient */
+  CS_PARAM_ITSOL_AMG,              /*!< Algebraic MultiGrid */
   CS_PARAM_N_ITSOL_TYPES
 
 } cs_param_itsol_type_t;
@@ -371,11 +407,11 @@ typedef enum {
 
 typedef struct {
 
-  cs_param_precond_type_t  precond; /*!< type of preconditioner */
-  cs_param_itsol_type_t    solver;  /*!< type of solver */
-
-  int          n_max_iter;          /*!< max. number of iterations */
-  double       eps;                 /*!< stopping criterion on accuracy */
+  cs_param_precond_type_t  precond;    /*!< type of preconditioner */
+  cs_param_itsol_type_t    solver;     /*!< type of solver */
+  cs_param_amg_type_t      amg_type;   /*!< type of AMG algorithm if needed  */
+  int                      n_max_iter; /*!< max. number of iterations */
+  double                   eps;        /*!< stopping criterion on accuracy */
 
   /*! \var resid_normalized
    *  normalized or not the norm of the residual used for the stopping criterion
@@ -418,6 +454,32 @@ cs_param_get_space_scheme_name(cs_param_space_scheme_t    scheme);
 const char *
 cs_param_get_time_scheme_name(cs_param_time_scheme_t    scheme);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Get the name of the type of boundary condition
+ *
+ * \param[in] type     type of boundary condition
+ *
+ * \return the associated bc name
+ */
+/*----------------------------------------------------------------------------*/
+
+const char *
+cs_param_get_bc_name(cs_param_bc_type_t  bc);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief   Get the name of the type of enforcement of the boundary condition
+ *
+ * \param[in] type          type of enforcement of boundary conditions
+ *
+ * \return the associated name
+ */
+/*----------------------------------------------------------------------------*/
+
+const char *
+cs_param_get_bc_enforcement_name(cs_param_bc_enforce_t  type);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief   Get the name of the solver
@@ -446,29 +508,16 @@ cs_param_get_precond_name(cs_param_precond_type_t  precond);
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief   Get the name of the type of boundary condition
+ * \brief   Get the name of the type of algebraic multigrid (AMG)
  *
- * \param[in] type     type of boundary condition
+ * \param[in] type     type of AMG
  *
- * \return the associated bc name
+ * \return the associated type name
  */
 /*----------------------------------------------------------------------------*/
 
 const char *
-cs_param_get_bc_name(cs_param_bc_type_t  bc);
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief   Get the name of the type of enforcement of the boundary condition
- *
- * \param[in] type          type of enforcement of boundary conditions
- *
- * \return the associated name
- */
-/*----------------------------------------------------------------------------*/
-
-const char *
-cs_param_get_bc_enforcement_name(cs_param_bc_enforce_t  type);
+cs_param_get_amg_type_name(cs_param_amg_type_t   type);
 
 /*----------------------------------------------------------------------------*/
 
