@@ -119,7 +119,7 @@ integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 double precision dt(ncelet)
 double precision gradv(3, 3, ncelet)
 double precision gradro(3,ncelet)
-double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
+double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
 double precision viscf(nfac), viscb(nfabor)
 double precision tslagi(ncelet)
 double precision smbr(ncelet), rovsdt(ncelet)
@@ -139,6 +139,8 @@ integer          ivar_r(3,3)
 integer          icvflb
 integer          init
 integer          ivoid(1)
+integer          key_t_ext_id
+integer          iroext
 
 double precision blencp, epsilp, epsrgp, climgp, extrap, relaxp
 double precision epsrsp
@@ -187,6 +189,11 @@ type(var_cal_opt) :: vcopt
 ! 1. Initialization
 !===============================================================================
 
+! Time extrapolation?
+call field_get_key_id("time_extrapolated", key_t_ext_id)
+
+call field_get_key_int(icrom, key_t_ext_id, iroext)
+
 ! Allocate work arrays
 allocate(w1(ncelet), w2(ncelet))
 allocate(dpvar(ncelet))
